//
//  The version of the class should be changed every time
//  when any changes in the codes are done
//  Grigory Nigmatkulov: 2016/12/15
//
//  Version 2 Nikita E:
//    Added transverse sphericity
//
#ifndef STO97DSTMAKER_H
#define STO97DSTMAKER_H


/*
 * General includes
 */

#include "StO97Track.h"
#include "StO97Event.h"
#include "StMaker.h"
#include "TTree.h"
#include "TFile.h"
#include "TChain.h"
#include "TSystem.h"
#include "TBranch.h"
#include "TDatime.h"
#include "TLorentzVector.h"

/*
 * Includes for transverse sphericity calculations
 */
#include <TVectorT.h>
#include <TMatrixTLazy.h>
#include <TMatrixDSymEigen.h>



//_________________
class StO97DstMaker : public StMaker {

 private:

  const Char_t *mOutFileName;
  const Char_t *mInFileName;
  TFile *mOutFile;
  FILE  *mInFile;
  TTree *mTree;
  Int_t  mCompression;

  Float_t mSperp;               // For primary tracks with pT>0.15 GeV and |eta|<0.5
  Float_t mSperp2;              // For primary tracks with pT>0.15 GeV and |eta|<1.0
  TMatrixTSym<double> *matrix;  // transverse sphericity matrix for mSperp
  TMatrixTSym<double> *matrix2; // transverse sphericity matrix for mSperp2

  Bool_t mStop;
  Int_t  mNBytes;
  Int_t  mNEvents;
  Int_t  mNTracks;
  char   mBuf[128]; // buffer for general read purposes

  StO97Event *mEvent;

  Bool_t ReadEvent();
  Bool_t ReadTracks();

 public:

  StO97DstMaker(const Char_t *iFileName,
                const Char_t *oFileName);
  ~StO97DstMaker();

  void  Clear(Option_t *option = "") { StMaker::Clear(option); }
  Int_t Init();
  Int_t Make();
  Int_t Finish();

  Bool_t IsStop() { return mStop; }
  void SetCompressionlevel(Int_t comp);

  ClassDef(StO97DstMaker, 2)
};

#endif
