#include "StPicoDstHbtReader.h"


ClassImp(StPicoDstHbtReader)


StPicoDstHbtReader::StPicoDstHbtReader(StPicoDstMaker *picoMaker,
                                       StRefMultCorr *refMultCorrUtil)
{
  std::cout << "Constructor" << std::endl;
  mPicoMaker       = picoMaker;
  mRefMultCorrUtil = refMultCorrUtil;
}


StPicoDstHbtReader::~StPicoDstHbtReader()
{
  //
  // NO-OP
  //
}


int StPicoDstHbtReader::Init()
{
  std::cout << "Init" << std::endl;

  return 0;
}


void StPicoDstHbtReader::Clear(Option_t *option) 
{
  /* NO-OP */
}


StHbtEvent *StPicoDstHbtReader::ReturnHbtEvent()
{
  nEventsProc++;

  StPicoDst *picoDst = mPicoMaker->picoDst();
  if (!picoDst)
  {
    std::cout << "can't obtain StPicoDst" << std::endl;
    return 0;
  }

  StPicoEvent *picoEvent = picoDst->event();

  if (!picoEvent)
  {
    std::cout << "can't obtain StPicoEvent" << std::endl;
    return 0;
  }

  StHbtEvent *hbtEvent = new StHbtEvent(); // create new hbt event
  //
  // Check trigger
  //
  unsigned int trig = (unsigned int)picoEvent->triggerWord();
  if (trig & (!mTrigWord))
    return hbtEvent;

  //
  // RefMultCorr Check and Init
  //  if mRefMultCorrUtil == 0 then data is not AuAu
  //
  StThreeVectorF mVertPosition(picoEvent->primaryVertex());
  int cent9 = -1;
  int cent16 = -1;
  double centWeight = -999.;
  double refMultCorrVal = -999.;

  if (mRefMultCorrUtil)
  { // AuAu collisions
    mRefMultCorrUtil->init(picoEvent->runId());
    if(mAuAuCorrectZdc) 
      mRefMultCorrUtil->initEvent(picoEvent->refMult(),
                                  mVertPosition.z(),
                                  0); // TODO: mMuEvent->runInfo().zdcCoincidenceRate());
    else
      mRefMultCorrUtil->initEvent(picoEvent->refMult(),
                                  mVertPosition.z());
    cent9  = mRefMultCorrUtil->getCentralityBin9();
    cent16 = mRefMultCorrUtil->getCentralityBin16();
    if(cent9 < 0 || cent16 < 0) 
      return 0;   // Some AuAu collisions have -1 value
    centWeight     = mRefMultCorrUtil->getWeight();
    refMultCorrVal = mRefMultCorrUtil->getRefMultCorr();
  }

  unsigned int nTracks = picoDst->numberOfTracks();


  //
  // Fill event
  //
  StThreeVectorF primVtxPos = picoEvent->primaryVertex();

  hbtEvent->SetEventNumber(picoEvent->runId());
  hbtEvent->SetRunNumber(picoEvent->runId());
  hbtEvent->SetZdcAdcEast(picoEvent->ZdcSumAdcEast());
  hbtEvent->SetZdcAdcWest(picoEvent->ZdcSumAdcWest());
  hbtEvent->SetUncorrectedNumberOfPositivePrimaries(picoEvent->refMultPos());
  hbtEvent->SetUncorrectedNumberOfNegativePrimaries(picoEvent->refMultNeg());
  hbtEvent->SetUncorrectedNumberOfPrimaries(picoEvent->refMult());
  hbtEvent->SetPrimVertPos(primVtxPos);
  hbtEvent->SetMagneticField(picoEvent->bField());
  hbtEvent->SetVpdVz(picoEvent->vzVpd());
  hbtEvent->SetCent9(cent9);
  hbtEvent->SetCent16(cent16);
  hbtEvent->SetRefMultCorr(refMultCorrVal);

  //
  // Fill tracks
  //
  for (unsigned int iTrack = 0; iTrack < nTracks; iTrack++)
  {
    StPicoTrack   *picoTrack = picoDst->track(iTrack);
    //
    // Get beta
    //
    float beta = picoTrack->btofBeta();

    //
    // Get DCA Geometry for DCA calculation
    //  and momentum with charge
    //
    float  p      = picoTrack->pMom().mag();
    short  charge = picoTrack->charge();
    StThreeVectorF origin = picoTrack->origin();

    StHbtTrack *hbtTrack = new StHbtTrack;
    //
    // Fill StHbtTrack
    //
    hbtTrack->SetTrackId(picoTrack->id());
    hbtTrack->SetFlag(300);

    unsigned int nHits = picoTrack->nHitsFit() - 1;
    hbtTrack->SetNHits(nHits*charge);
    hbtTrack->SetNHitsPossible(picoTrack->nHitsMax());
    hbtTrack->SetNHitsDedx(picoTrack->nHitsDedx());

    hbtTrack->SetNSigmaElectron(picoTrack->nSigmaElectron());
    hbtTrack->SetNSigmaPion(picoTrack->nSigmaPion());
    hbtTrack->SetNSigmaKaon(picoTrack->nSigmaKaon());
    hbtTrack->SetNSigmaProton(picoTrack->nSigmaProton());

    hbtTrack->SetPidProbElectron(0.5);
    hbtTrack->SetPidProbPion(0.5);
    hbtTrack->SetPidProbKaon(0.5);
    hbtTrack->SetPidProbProton(0.5);

    hbtTrack->SetdEdx(picoTrack->dEdx()*1e6);
    hbtTrack->SetChi2(picoTrack->chi2());

    StPhysicalHelixD globHelix(picoTrack->gMom(),
                               origin,
                               picoEvent->bField() * kilogauss,
                               charge);
    hbtTrack->SetHelixGlobal(globHelix);

    StPhysicalHelixD primHelix(picoTrack->pMom(),
                               primVtxPos,
                               picoEvent->bField() * kilogauss,
                               charge);
    hbtTrack->SetHelix(primHelix);

    CalcDca(globHelix, primVtxPos);
    hbtTrack->SetDCAxyGlobal(mDca.perp());
    hbtTrack->SetDCAzGlobal(mDca.z());
    hbtTrack->SetP(picoTrack->pMom());

    unsigned int map0 = 0, map1 = 0;
    for (unsigned int iMap = 0; iMap < 64; iMap++) 
    {
      if (iMap < 8)
        map0 |= 1 << iMap;
      else
      {
        if (iMap < 32)
        {
          if ((iMap - 8) < nHits)
            map0 |= 1 << iMap;
          else
            map0 |= 0 << iMap;
        }
        else
        {
          if (iMap < 53)
          {
            if ((iMap - 8) < nHits)
              map1 |= 1 << iMap;
            else
              map1 |= 0 << iMap;
          }
          else
          {
            if (iMap == 55)
              if (beta >= 0.)
                map1 |= 1 << iMap;
              else
                map1 |= 0 << iMap;
            else
              map1 |= 1 << iMap;
          }
        }
      }
    }

    hbtTrack->SetTopologyMap(0, map0);
    hbtTrack->SetTopologyMap(1, map1);

    hbtTrack->SetTofBeta(beta);
    hbtTrack->SetHiddenInfo(0);

    hbtEvent->TrackCollection()->push_back(hbtTrack);
  }

  return hbtEvent;
}


void StPicoDstHbtReader::Finish()
{
  std::cout << "Finish!" << std::cout;
}


void StPicoDstHbtReader::CalcDca(const StPhysicalHelixD &helix,
                                 const StThreeVectorF   &vtxPos)
{
  double pathLen = helix.pathLength(vtxPos, false);
  mDca = helix.at(pathLen) - vtxPos;
}
