/****************************************/
/***                                  ***/
/***          StPicoDstHbtReader      ***/
/***  VERSION: 1.0                    ***/
/***   AUTHOR: Nikita Ermakov         ***/
/***     DATE:                        ***/
/***   E-MAIL: coffe92@gmail.com      ***/
/***                                  ***/
/****************************************/

#ifndef STPICODSTQA

#include "StPicoDst.h"
#include "StPicoDstMaker.h"
#include <stdio.h>
#include "TH2F.h"
#include "TH1F.h"
#include "StPicoEvent.h"
#include "StPicoTrack.h"
#include "StPicoBTofPidTraits.h"
#include "StHbtMaker/Base/StHbtEventReader.hh"
#include "StHbtMaker/Infrastructure/StHbtEvent.hh"
#include "StHbtMaker/Infrastructure/StHbtEnumeration.hh"
#include "StEvent/StEnumerations.h"
#include "StRefMultCorr/StRefMultCorr.h"


class StPicoDst;
class StPicoEvent;
class StPicoTrack;


class StPicoDstHbtReader : public StHbtEventReader
{
 public:
  StPicoDstHbtReader(StPicoDstMaker *picoMaker,
                     StRefMultCorr  *refMultCorrUtil = 0);
  ~StPicoDstHbtReader();

  void SetAuAuZdcCoincidenceEnergy(bool zdcCorr) { mAuAuCorrectZdc = zdcCorr; }

  void SetTriggers(unsigned int trigWord) { mTrigWord = trigWord; }
  //
  // Chain methods
  //
  void  Clear(Option_t *option = "");
  int Init();
  int Init(const char* ReadWrite, StHbtString &Message)
          { return StHbtEventReader::Init(ReadWrite, Message); }
 
  StHbtEvent *ReturnHbtEvent();
  void Finish();
 private:
  void CalcDca(const StPhysicalHelixD &helix,
               const StThreeVectorF   &vtxPos);

  TChain         *mChain;
  TTree          *mTree;
  StPicoDstMaker *mPicoMaker;
  StHbtEvent     *mHbtEvent;
  StRefMultCorr  *mRefMultCorrUtil;
  bool            mAuAuCorrectZdc;
  StThreeVectorF  mDca;

  unsigned int mTrigWord;

  Long64_t nEvents;
  Long64_t nEventsProc;


  ClassDef(StPicoDstHbtReader, 1)
};

#endif
