/*
 * READ ME:
 *   For proper working of this maker you need to link StRoot to your directory.
 *   For more info please take a look on StO97DstQAMaker.cxx line 37
 *
 */

#ifndef STQGSMDSTQAMAKER_HH
#define STQGSMDSTQAMAKER_HH


#include <iostream>
#include "StMaker.h"
#include "StChain.h"
#include "TChain.h"
#include "TSystem.h"
#include "TH1.h"
#include "TH2.h"
#include "StFemtoDstMaker/StO97Event.h"
#include <string>
#include <stdlib.h>
#include "TDatabasePDG.h"


//_________________
class StQGSMdstQAMaker : public StMaker {
public:
  StQGSMdstQAMaker(const char *dirName,
                   const char *fileName,
                   const char *filter = ".",
                   int        maxFiles = 1e5);
  int GetNEvents();

  Int_t Init();
  Int_t Make();
  Int_t Finish();
  void SetOutFileName(const char *oFileName) { mOutFileName = oFileName; }

private:
  string mDir;
  string mFileName;
  string mFilter;

  TDatabasePDG *pdgDb;
  unsigned int mEventId;
  unsigned int mNEvents;
  TFile        *mOutFile;
  TChain       *mChain;
  TTree        *mTree;
  StO97Event   *mO97Event;
  const char   *mOutFileName;
  int           mMaxFiles;

  //
  // Event histograms
  //
  TH1F *hRefMultP90;
  TH1F *hRefMultP100;
  TH1F *hRefMultP120;
  TH1F *hRefMultP150;
  TH1F *hImpPar;
  TH1F *hImp;
  TH1F *hSph;
  TH1F *hSph2;
	TH2F *hImpVsRef;

  //
  // Track histograms
  //
  TH2F *hMSqrVsPt;
  TH1F *hVx;
  TH1F *hVy;
  TH1F *hVz;
  TH1F *hP;
  TH1F *hPt;
  TH1F *hPx;
  TH1F *hPy;
  TH1F *hPz;
  TH1F *hEta;
  TH1F *hEtaInt;
  TH1F *hMomElectron;
  TH1F *hMomPion;
  TH1F *hMomKaon;
  TH1F *hMomProton;

  TH2F *hRfrVsTfr_pion;
  TH2F *hRfrVsTfr_kaon;
  TH2F *hRfrVsTfr_proton;

  TH1F *hdNdTfr_pion;
  TH1F *hdNdTfr_kaon;
  TH1F *hdNdTfr_proton;

  int InitRead(string dir, string fileName,
               string filter, int maxFiles);
  int FillChain(TChain *chain, const char *fileName, 
                int maxFiles);
  int FillChain(TChain *chain, char *dir,
                const char *filter, int maxFiles);
  void UninitRead();

  ClassDef(StQGSMdstQAMaker, 1)
};

#endif
