#ifndef STSPHERICITY_HH
#define STSPHERICITY_HH

#include "StMaker.h"
#include "StThreeVector.hh"
#include "StThreeVectorF.hh"
#include "StPhysicalHelixD.hh"
#include "StRefMultCorr/StRefMultCorr.h"

#include "StMuDSTMaker/COMMON/StMuDstMaker.h"
#include "StMuDSTMaker/COMMON/StMuDst.h"
#include "StMuDSTMaker/COMMON/StMuEvent.h"
#include "StMuDSTMaker/COMMON/StMuTrack.h"
#include "StMuDSTMaker/COMMON/StMuPrimaryVertex.h"
#include "StMuDSTMaker/COMMON/StMuBTofPidTraits.h"
#include "StMuDSTMaker/COMMON/StMuBTofHit.h"
#include "StBTofHeader.h"

#include <TString.h>
#include <TFile.h>
#include <TLorentzVector.h>
#include <TTree.h>
#include <TH1.h>
#include <TH2.h>
#include <TProfile.h>
#include <TVectorT.h>
#include <TMatrixTLazy.h>
#include <TMatrixDSymEigen.h>

#include <vector>
#include <iostream>

using namespace std;

class StMuDst;
class StMuDstMaker;
class StMuEvent;
class StMuTrack;


class StSphericity : public StMaker
{
    public:
     StSphericity(StMuDstMaker *muMaker,
                  const Char_t *oFileName);
     ~StSphericity();

     void AddTrigId(unsigned int aTrigId);

     void  Clear(Option_t *option = "");
     Int_t Init();
     Int_t Make();
     Int_t Finish();

     void SetMuDstMaker(StMuDstMaker *maker);
     void SetOutFileName(const char *oFileName) {mOutFileName = oFileName;}
     void SetSphericityLowCut(float val) { sphLoCut = val; }
     void SetSphericityHighCut(float val) { sphHiCut = val; }
     void SetPtCut(float pt) { ptCut = pt; }
     bool AcceptTrack(StMuTrack *track);

    private:
     TFile        *mOutFile;
     const char   *mOutFileName;
     unsigned int  mNEventsIn;

     StMuPrimaryVertex    *mPrimVertex;
     StMuTrack            *mPrimTrack;
     StMuTrack            *mGlobTrack;
     StMuDstMaker         *mMuDstMaker;
     StMuEvent            *mMuEvent;
     StMuDst              *mMuDst;
     float                 ptCut;
     TMatrixTSym<double>  *matrix;    // transverse sphericity matrix
     TMatrixTSym<double>  *matrix3d;  // 3D sphericity matrix
     float                 sphLoCut, sphHiCut;

     std::vector<unsigned int> mTrigIdCol;

     TH1I *hRefMult;

     TH1F *hEta;
     TH1F *hPhi;
     TH2F *hEtaPhi;
     TH2F *hDEtaDPhi;
     //
     // Manual calculation of the sphericity
     //
     TH1F *hSphManual;
     TH2F *hDEtaDPhi_lo_sphManual; // sphericity < lo "jet-like" events
     TH2F *hDEtaDPhi_hi_sphManual; // sphericity > hi isotropic
     //
     // Automatic calculation of the sphericity through the ROOT
     //
     TH1F *hSphAuto;
     TH2F *hDEtaDPhi_lo_sphAuto; // sphericity < lo "jet-like" events
     TH2F *hDEtaDPhi_hi_sphAuto; // sphericity > hi isotropic
     //
     // 3D sphericity
     //
     TH1F *hSph3d;
     TH2F *hDEtaDPhi_lo_sph3d; // sphericity < lo "jet-like" events
     TH2F *hDEtaDPhi_hi_sph3d; // sphericity > hi isotropic

     TH1F *hSphDiff;   // difference between manual and auto calculated sphericity
     TH1F *hSph3dDiff; // sphAuto - sph3d

     ClassDef(StSphericity, 1)
};

#endif
