#ifndef STSPHERICITY_HH
#define STSPHERICITY_HH

#include <iostream>
#include "StMaker.h"
#include "StChain.h"
#include "TChain.h"
#include "TSystem.h"
#include "TH1.h"
#include "TH2.h"
#include "TH3.h"
#include "TProfile.h"
#include "StFemtoDstMaker/StFemtoEvent.h"
#include <string>
#include <stdlib.h>

class StSphericity : public StMaker
{
    public:
        StSphericity(const char *aDirName, 
                     const char *aFileName,
                     const char *aFilter = ".", 
                     int aMaxFiles = 10000);
        void AddTrigId(unsigned int aTrigId);
        int GetNEvents();

        Int_t Init();
        Int_t Make();
        Int_t Finish();
        void SetOutFileName(const char *oFileName) {outFileName = oFileName;}
        void SetTrigger(unsigned int trig, char mode);

    private:
        string dir;
        string fileName;
        string filter;

        unsigned int eventId;
        unsigned int nEvents;
        TFile        *outFile;
        TChain       *chain;
        TTree        *tree;
        StFemtoEvent *femtoEvent;
        const char   *outFileName;

        std::vector<unsigned int> mTrigIdCol;
        unsigned int              mTrig;
    
        TH1F *hEta;
        TH1F *hPhi;
        TH2F *hEtaPhi;
        TH2F *hDEtaDPhi;
        TH3F *hDEtaDPhiNPairs;
        TH2F *hDEtaDPhi_lo_sph; // sphericity < 0.1 "jet-like" events
        TH2F *hDEtaDPhi_hi_sph; // sphericity > 0.9 isotropic
        TH1F *hSphericity;

        int maxFiles;

        int InitRead(string aDir, string aFileName,
                string aFilter, int aMaxFiles);
        int FillChain(TChain *aChain, const char *aFileName, int aMaxFiles);
        int FillChain(TChain *aChain, char *aDir, const char *aFilter, int aMaxFiles);
        void UninitRead();

        ClassDef(StSphericity, 1)
};

#endif
