#include "StSphericity.h"
#include "StThreeVector.hh"
#include "StThreeVectorF.hh"
#include "StPhysicalHelixD.hh"

ClassImp(StSphericity)


StSphericity::StSphericity(const char *aDir, 
                           const char *aFileName,
                           const char *aFilter,
                           int aMaxFiles)
{
    dir      = string(aDir);
    fileName = string(aFileName);
    filter   = aFilter;
    maxFiles = aMaxFiles;
    outFileName = "sphManual.root";

    tree     = 0;
    chain    = 0;
    event    = 0;
    eventId  = 0;
    pCut     = 0.5;
    matrix   = new TMatrixTSym<double>(2);
    matrix3d = new TMatrixTSym<double>(3);
    sphLoCut = 0.1;
    sphHiCut = 0.8;
}



StSphericity::~StSphericity()
{
    delete matrix;
}



Int_t StSphericity::Init()
{
    InitRead(dir, fileName, filter, maxFiles);
    nEvents = (unsigned int)chain->GetEntries();
    // create an output file
    outFile = new TFile(outFileName, "RECREATE");
    //
    // Create histograms
    //
    // no cut to sphericity
    hEta = new TH1F("hEta", ";#eta;#", 100, -3., 3.);
    hPhi = new TH1F("hPhi", ";#phi [rad];#", 300, -7., 7.);
    hDEtaDPhi = new TH2F("hDEtaDPhi", ";#Delta#eta;#Delta#phi [rad]",
                         200, -3., 3., /* eta */
                         400, -1.5, 6. /* phi */);
    hEtaPhi = new TH2F("hEtaPhi", ";#eta;#phi [rad]", 
                       200, -3., 3., /* eta */
                       400, -1.5, 6. /* phi */);
    // with sphericity cut
    hDEtaDPhi_lo_sphManual = new TH2F("hDEtaDPhi_lo_sphManual", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    hDEtaDPhi_hi_sphManual = new TH2F("hDEtaDPhi_hi_sphManual", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    hDEtaDPhi_lo_sphAuto = new TH2F("hDEtaDPhi_lo_sphAuto", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    hDEtaDPhi_hi_sphAuto = new TH2F("hDEtaDPhi_hi_sphAuto", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    hDEtaDPhi_lo_sph3d = new TH2F("hDEtaDPhi_lo_sph3d", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    hDEtaDPhi_hi_sph3d = new TH2F("hDEtaDPhi_hi_sph3d", ";#Delta#eta;#Delta#phi [rad]",
                                      200, -3., 3., /* eta */
                                      400, -1.5, 6.  /* phi */);
    // transverse and 3d sphericity histograms
    hSphManual = new TH1F("hSphManual", ";Sphericity;#",
                          300, -0.1, 1.1);
    hSphAuto = new TH1F("hSphAuto", ";Sphericity;#",
                        300, -0.1, 1.1);
    hSphDiff = new TH1F("hSphDiff", ";Sphericity_{auto} - Sphericity_{manual};#",
                        200, -1., 1.);
    hSph3dDiff = new TH1F("hSph3dDiff", ";Sphericity_{auto} - Sphericity_{3d};#",
                        200, -1., 1.);
    hSph3d = new TH1F("hSph3d", ";Sphericity_{3D};#",
                      300, -0.1, 1.1);
    // momntum distributions with S_transverse cut
    hP3DLow = new TH3F("hP3DLow", ";p_{x} [GeV/c];p_{y} [GeV/c];p_{z} [GeV/c]",
                       100, -5., 5., /* px */
                       100, -5., 5., /* py */
                       100, -5., 5.  /* pz */);
    hPxPyLow = new TH2F("hPxPyLow", ";p_{x} [GeV/c]; p_{y} [GeV/c]",
                        500, -5., 5., /* px */
                        500, -5., 5.  /* py */);
    hPxPzLow = new TH2F("hPxPzLow", ";p_{x} [GeV/c]; p_{z} [GeV/c]",
                        500, -5., 5., /* px */
                        500, -5., 5.  /* pz */);
    hPyPzLow = new TH2F("hPyPzLow", ";p_{y} [GeV/c]; p_{z} [GeV/c]",
                        500, -5., 5., /* py */
                        500, -5., 5.  /* pz */);
    hP3DHigh = new TH3F("hP3DHigh", ";p_{x} [GeV/c];p_{y} [GeV/c];p_{z} [GeV/c]",
                       100, -5., 5., /* px */
                       100, -5., 5., /* py */
                       100, -5., 5.  /* pz */);
    hPxPyHigh = new TH2F("hPxPyHigh", ";p_{x} [GeV/c]; p_{y} [GeV/c]",
                        500, -5., 5., /* px */
                        500, -5., 5.  /* py */);
    hPxPzHigh = new TH2F("hPxPzHigh", ";p_{x} [GeV/c]; p_{z} [GeV/c]",
                        500, -5., 5., /* px */
                        500, -5., 5.  /* pz */);
    hPyPzHigh = new TH2F("hPyPzHigh", ";p_{y} [GeV/c]; p_{z} [GeV/c]",
                        500, -5., 5., /* py */
                        500, -5., 5.  /* pz */);
    // momntum distributions without S_transverse cut
    hP3D_wocut = new TH3F("hP3D_wocut", ";p_{x} [GeV/c];p_{y} [GeV/c];p_{z} [GeV/c]",
                          100, -5., 5., /* px */
                          100, -5., 5., /* py */
                          100, -5., 5.  /* pz */);
    hPxPy_wocut = new TH2F("hPxPy_wocut", ";p_{x} [GeV/c]; p_{y} [GeV/c]",
                           500, -5., 5., /* px */
                           500, -5., 5.  /* py */);
    hPxPz_wocut = new TH2F("hPxPz_wocut", ";p_{x} [GeV/c]; p_{z} [GeV/c]",
                           500, -5., 5., /* px */
                           500, -5., 5.  /* pz */);
    hPyPz_wocut = new TH2F("hPyPz_wocut", ";p_{y} [GeV/c]; p_{z} [GeV/c]",
                           500, -5., 5., /* py */
                           500, -5., 5.  /* pz */);

    return kStOk;
}



Int_t StSphericity::Finish()
{
    UninitRead();

    outFile->Write();
    outFile->Close();

    return kStOk;
}



int StSphericity::FillChain(TChain *aChain, char *aDir,
                            const char *aFilter, int aMaxFiles)
{
    TSystem *gSystem = new TSystem();
    if (!gSystem)
    {
        std::cout << "StSphericity[ERROR]: can't allocate memory for TSystem\n";
        return 0;
    }
    void *lDir = gSystem->OpenDirectory(aDir);
    if (!lDir)
    {
        std::cout << "StSphericity[ERROR]: can't open directory " 
            << aDir << std::endl;
        delete gSystem;
        return 0;
    }

    int lCount = 0;
    char *lFileName;

    while ((lFileName = (char *)gSystem->GetDirEntry(aDir)))
    {
        if (strcmp(lFileName, ".") == 0 || strcmp(lFileName, "..") == 0)
            continue;
        if (strstr(lFileName, aFilter))
        {
            char *lFullName = gSystem->ConcatFileName(aDir, lFileName);
            std::cout << "StSphericity[INFO]: Adding "
                << lFullName << " to the chain" << std::endl;
            aChain->Add(lFullName);
            delete lFullName;
            lCount++;
            if ((aMaxFiles > 0) && (lCount > aMaxFiles))   break;
        }
    }

    std::cout << "StSphericity[INFO]: Added " << lCount
        << " files to the chain" << std::endl;
    delete gSystem;
    return lCount;
}



int StSphericity::FillChain(TChain *aChain, const char *aFileName,
                            int aMaxFiles)
{
    std::ifstream lInStream(aFileName);

    if (!lInStream.is_open())
    {
        std::cout << "StSphericity[ERROR]: can't open file list: "
            << aFileName << std::endl;
        return 0;
    }
    std::cout << "StSphericity[INFO]: using file list: "
        << aFileName << std::endl;

    int lCount = 0;
    char buf[0xFF];
    for ( ; lInStream.good(); )
    {
        lInStream.getline(buf, 0xFF);
        TString lFileName(buf);
        if (lFileName.Contains("root"))
        {
            aChain->Add(buf);
            lCount++;
            if ((aMaxFiles > 0) && (lCount > aMaxFiles)) break;
        }
    }

    std::cout << "StSphericity[INFO]: Added " << lCount
        << " files to the chain" << std::endl;
    return lCount;
}



int StSphericity::InitRead(string aDir, string aFileName,
                           string aFilter, int aMaxFiles)
{
    eventId = 0;
    chain = new TChain("genevents", "genevents");

    std::cout << "StSphericity[INFO]: Call InitRead" << std::endl;

    int lNumFiles = 0;
    if (!aFileName.empty())
    {
        if (strstr(aFileName.c_str(), ".lis") ||
            strstr(aFileName.c_str(), ".list"))
        {
            lNumFiles = FillChain(chain, 
                                  (aDir + aFileName).c_str(), 
                                  aMaxFiles);
        }
        else
        {
            chain->Add((aDir + aFileName).c_str());
            lNumFiles++;
        }
    }
    else
    {
        lNumFiles = FillChain(chain, (char *)aDir.c_str(), // not cool
                              aFilter.c_str(), aMaxFiles);
    }
    chain->SetBranchAddress("primaryEvent", &event);
    nEvents = (unsigned int)chain->GetEntries();
    return lNumFiles;
}




void StSphericity::UninitRead()
{
    if (chain)   delete chain;
    event = 0;
    chain = 0;
}



int StSphericity::GetNEvents()
{
    if (chain)
        return nEvents;
    else
        return -1;
}




Int_t StSphericity::Make()
{
    if (!nEvents)
    {
        std::cout << "StSphericity[INFO]: no events to read\n";
        return 0;
    }

    int lBytes = chain->GetEntry(eventId++);

    if (nEvents < eventId)
    {
        std::cout << "StSphericity[INFO]: EOF\n";
        return 0;
    }

    if (!lBytes)
    {
        std::cout << "StSphericity[INFO]: no event\n";
        return 0;
    }

    if (event)
    {
        int lNTracks = event->GetNumberOfParticles();

        if (lNTracks > 1)
        {
            int numOfLeadPart = -1;
            float pOfLeadPart = -999.;
            float sphManual   = -999.;
            float sphAuto     = -999.;
            float sph3d       = -999.;
            float pt_full     = 0.0;
            float p_full      = 0.0;
            // (a       m_diag)
            // (m_diag  b     )
            float m_a = 0., m_diag = 0., m_b = 0.;
            float D = 0.0;   // discriminant
            float lambda[2]; // eigenvalues
            unsigned int nPairs = 0;

            // fill merticies with zeros
            matrix->Zero();
            matrix3d->Zero();

            for (int i = 0; i < lNTracks; i++)
            {
                StarGenParticle *lTrack = (*event)[i];
                const TLorentzVector &mom = lTrack->momentum();
                float pt   = mom.Pt();
                float ptot = mom.Mag();
                float px   = mom.Px();
                float py   = mom.Py();
                float pz   = mom.Pz();

                // fill momentum distributions
                hP3D_wocut->Fill(px, py, pz);
                hPxPy_wocut->Fill(px, py);
                hPxPz_wocut->Fill(px, pz);
                hPyPz_wocut->Fill(py, pz);

                if (ptot > pCut)
                {
                    if (pt > 1e-10)
                    {
                        if (fabs(mom.Eta()) < 1.0)
                        {
                            if (pOfLeadPart < ptot)
                            {
                                numOfLeadPart = i;
                                pOfLeadPart = ptot;
                            }
                            float buf_a = px*px/pt;
                            float buf_b = py*py/pt;
                            float buf_diag = px*py/pt;

                            // fill matrix for transverse sphericity
                            //   (row, col)
                            (*matrix)(0, 0) += buf_a;
                            (*matrix)(1, 1) += buf_b;
                            (*matrix)(0, 1) += buf_diag;
                            (*matrix)(1, 0) += buf_diag;
                            // manual calcs
                            m_a += buf_a;
                            m_b += buf_b;
                            m_diag += buf_diag;
                            pt_full += pt;
                            nPairs++;
                        }
                    }
                    // fill matrix for 3D sphericity
                    //   (row, col)
                    (*matrix3d)(0, 0) += px*px;
                    (*matrix3d)(0, 1) += px*py;
                    (*matrix3d)(0, 2) += px*pz;

                    (*matrix3d)(1, 0) += py*px;
                    (*matrix3d)(1, 1) += py*py;
                    (*matrix3d)(1, 2) += py*pz;

                    (*matrix3d)(2, 0) += pz*px;
                    (*matrix3d)(2, 1) += pz*py;
                    (*matrix3d)(2, 2) += pz*pz;
                    p_full += ptot;
                }
            }

            if (numOfLeadPart < 0 || nPairs < 2)   return kStOk;

            //
            // Normalize matrices
            //
            m_a /= pt_full;
            m_b /= pt_full;
            m_diag /= pt_full;
            *matrix *= 1./pt_full;
            *matrix3d *= 1./(p_full*p_full);

            //
            // Calculate eigenvalues
            //
            D = pow(m_a + m_b, 2.) - 4.*(m_a*m_b - m_diag*m_diag);
            if (D >= 0.) // two real lambda
            {
                lambda[0] = 0.5*(m_a + m_b + sqrt(D));
                lambda[1] = 0.5*(m_a + m_b - sqrt(D));
                sphManual = 2./(lambda[0] + lambda[1]);
                if (lambda[0] < lambda[1])
                    sphManual *= lambda[0];
                else
                    sphManual *= lambda[1];
                hSphManual->Fill(sphManual);
            }

            //
            // Automatic calculation of eigenvalues
            //
            // transverse sphericity
            TMatrixDSymEigen death_machine(*matrix);
            TVectorD eigen = death_machine.GetEigenValues();
            sphAuto = 2.*eigen.Min()/(eigen[0] + eigen[1]);
            hSphAuto->Fill(sphAuto);
            hSphDiff->Fill(sphAuto - sphManual);
            // 3D sphericity
            TMatrixDSymEigen death_machine_3d(*matrix3d);
            TVectorD eigen3d = death_machine_3d.GetEigenValues();
            float l1 = eigen3d.Min();
            float l2 = l1;
            float lmax = eigen3d.Max();
            for (int i = 0; i < 3; i++)
            {
                if (eigen3d[i] > l1 && eigen3d[i] < lmax)
                {
                    l2 = eigen3d[i];
                    break;
                }
            }
            sph3d = 3./2.*(l1 + l2);
            hSph3d->Fill(sph3d);
            hSph3dDiff->Fill(sphAuto - sph3d);

            StarGenParticle *leadTrack = (*event)[numOfLeadPart];
            const TLorentzVector &momLead = leadTrack->momentum();
            for (int i = 0; i < lNTracks; i++)
            {
                if (i == numOfLeadPart)   continue;
                StarGenParticle *lTrack = (*event)[i];
                const TLorentzVector &mom = lTrack->momentum();
                float ptot = mom.Mag();
                float px   = mom.Px();
                float py   = mom.Py();
                float pz   = mom.Pz();
                float pt   = mom.Pt();
                if (ptot > pCut && pt > 1e-10)
                {
                    float phi = mom.Phi();
                    float eta = mom.Eta();
                    float dEta = eta - momLead.Eta();
                    float dPhi = phi - momLead.Phi();

                    hEta->Fill(eta);
                    hPhi->Fill(phi);
                    hEtaPhi->Fill(eta, phi);
                    hDEtaDPhi->Fill(dEta, dPhi);
                    // manual calculated sphericity
                    if (sphManual >= 0. && sphManual < sphLoCut)
                        hDEtaDPhi_lo_sphManual->Fill(dEta, dPhi);
                    if (sphManual > sphHiCut && sphManual <= 1.0)
                        hDEtaDPhi_hi_sphManual->Fill(dEta, dPhi);
                    // automatic calculated sphericity
                    if (sphAuto >= 0. && sphAuto < sphLoCut)
                    {
                        hDEtaDPhi_lo_sphAuto->Fill(dEta, dPhi);
                        hP3DLow->Fill(px, py, pz);
                        hPxPyLow->Fill(px, py);
                        hPxPzLow->Fill(px, pz);
                        hPyPzLow->Fill(py, pz);
                    }
                    if (sphAuto > sphHiCut && sphAuto <= 1.0)
                    {
                        hDEtaDPhi_hi_sphAuto->Fill(dEta, dPhi);
                        hP3DHigh->Fill(px, py, pz);
                        hPxPyHigh->Fill(px, py);
                        hPxPzHigh->Fill(px, pz);
                        hPyPzHigh->Fill(py, pz);
                    }
                }
            }
        }
    }
    return kStOk;
}
