//
// In this macro I change only upper limits of particle momentum
//
#include <TROOT.h>
#include <TSystem.h>
#include <TChain.h>
#include <TFile.h>
#include <TString.h>
#include <iostream>

//
// To define cut monitors
// uncomments line below
//
#define MONITORS

//
// Constants for analyses
//

#define CHARGE_BINS 2
#define SPH_BINS 10

//
//  Event information
//
const int mPrimVertZbins = 6;                // number of z bins for analysis
const double mPrimVertZ[2] = {-30., 30.};    // redefine values of the vertex Z for each run [cm]
const double mPrimVertR = 2.;                // distance in the XY plane [cm]
const double mPrimVertVpdVz[2] = {-5., 5.};  // VpdZ - TPCZ difference
const double mPrimVertXShift = 0.;
const double mPrimVertYShift = 0.;
const bool   mUseZdcCorrection = false;      // valid for AuAu collisions. false for 200GeV only!
const float  mKRange[2] = {0.25, 10.}; // ref=k*tof + b
const float  mBRange[2] = {-10., 0.};  //

//
// Track information
//
const double mChargedKaonMass = 0.493677;
const double mChargedPionMass = 0.13957018;
const double mChargedProtonMass = 0.938272;
const int mPosCharge = +1;
const int mNegCharge = -1;
const int mTrackType = 1;                     // primary track
const int mPythiaKaonCode = 321;              // pythia particle code
const int mPythiaPionCode = 211;              // pythia particle code
const int mPythiaProtonCode = 2212;           // pythia particle code

const int mTrackNHits[2] = {10, 50};          // number of track hits
const double mTrackNHitsFit[2] = {5, 50};    // number of track hits used in fit
const double mTrackDCA[2] = {0., 2.};         // DCA of the track and PV [cm]
const double mTrackASplitVal = 0.;
const double mTrackMom[2] = {0.12, 1.55};        // general momentum of the track [GeV/c]
const double mTrackTransvMom[2] = {0.05, 1.55};  // general transverse momentum of the track [GeV/c]
const double mTrackEta[2] = { -0.5, 0.5};

const short mDetectorSelection = 3;  // 0-TPC, 1-ToF, 2-TPC+ToF, 3-ToF||TPC

const double mTrackTpcMom[2] = {0.15, 0.55};  // track momentum for TPC idnetification [GeV/c]
const double mnSigmaElectron[2] = {-1e9, 1e9};
const double mnSigmaPion[2] = {-1e9, 1e9};
const double mnSigmaKaon[2] = {-1e9, 1e9};
const double mnSigmaProton[2] = {-2., 2.};

const double mTrackTofMom[2] = {0.15, 1.45};   // track momentum for ToF identification [GeV/c]
const double mPionTofMsqr[2] = {-0.02, 0.062}; // based on p+p data
const double mKaonTofMsqr[2] = {0.2, 0.32};    // based on p+p data
const double mProtonTofMsqr[2] = {0.7, 1.2};   // based on p+p data

const double mTrackTnTMom[2] = {0.15, 1.55};
const double mTnTNSigmaElectron[2] = {-1e9, 1e9};
const double mTnTNSigmaPion[2] = {-1e9, 1e9};
const double mTnTNSigmaKaon[2] = {-1e9, 1e9};
const double mTnTNSigmaProton[2] = {-2., 2.};

//
// Pair information
//
const double mFmrCut[2] = {-1.1, 0.1};         // not more than 10% of merged rows
const double mSplitLevelCut[2] = {-0.5, 0.6};  // splitting level from paper. StHbtPair::quality()
const double mPairKt[2] = {-1e9., 1e9};        // pair transverse momentum cut
const double mPairQinv[2] = {-1e9., 1e9};      // pair relative momentum
const double mAveSep[2] = {5., 5000.};         // average separation of tracks within TPC

//
// Histograms values
//
const int mPairKtBins = 28;                // bin width
const double mPairKtVal[2] = {0.1, 1.5};   // 50 MeV/c
const int mQinvBins = 150;                 // 20 MeV/c^2 bin width
const double mQinvVal[2] = {0., 3.0};      // redefine for each analysis

const Char_t *defaultInFile = "test.list";
const Char_t *defaultOutFile = "oSimProton_pp200_1D.root";



void sim97ProtonHBT_pp200n510_1D(const Char_t *inFileList = defaultInFile,
																 const Char_t *oFileName  = defaultOutFile)
{
				std::cout << "*****************************" << std::endl
									<< "*    simProtonHBT_pp200_1D  *" << std::endl
									<< "*          Start            *" << std::endl
									<< "*****************************" << std::endl << std::endl;

				//
				// Load libraries
				//
				std::cout << "Loading libraries..." << std::endl;
				gROOT->LoadMacro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
				loadSharedLibraries();
				gSystem->Load("libMinuit");
				gSystem->Load("StRefMultCorr");
				gSystem->Load("StFlowMaker");   // should be placed before HBT
				gSystem->Load("StHbtMaker");
				gSystem->Load("StarClassLibrary");
				gSystem->Load("libgsl");
				gSystem->Load("libgslcblas");
				gSystem->Load("StPicoDstMakerRun12");
				gSystem->Load("StarClassLibrary");
				gSystem->Load("libgsl");
				gSystem->Load("libgslcblas");
				gSystem->Load("libVMC.so");
				gSystem->Load("libStDb_Tables.so");
				gSystem->Load("libgen_Tables.so");
				gSystem->Load("libgeometry_Tables.so");
				gSystem->Load("libsim_Tables.so");
				gSystem->Load("libStarMagField.so");
				gSystem->Load("libSt_db_Maker.so");
				gSystem->Load("libSt_g2t.so");
				gSystem->Load("libSt_geant_Maker.so");
				gSystem->Load("StarGeneratorUtil.so");
				gSystem->Load("StarGeneratorEvent.so");
				gSystem->Load("StarGeneratorBase.so");
				gSystem->Load("StFemtoDstMaker");

				std::cout << "Libraries have been successfully loaded" << std::endl;

				//
				// Create chain
				//
				StChain* mChain = new StChain("StChain");
				mChain->SetDebug(0);
				StMuDebug::setLevel(0);

				//
				// Set StHbtMaker and StHbtManager
				//
				StHbtMaker *mHbtMaker = new StHbtMaker();
				mHbtMaker->SetDebug(0);
				StHbtManager *mHbtManager = mHbtMaker->HbtManager();

				//
				// Set StHbtReader
				//
				StHbtO97DstReader *o97Reader = new StHbtO97DstReader("", inFileList, "root", -1, false);

				Long64_t mNEvents = o97Reader->GetNEvents();
				mHbtManager->SetEventReader(o97Reader);

				//
				// Set basic event cut
				//
				gregsEventCut *mBasicEventCut = new gregsEventCut();
				mBasicEventCut->SetCollisionType(0);     //0-pp, 1-AuAu
				mBasicEventCut->SetEventMult(0, 45);
				mBasicEventCut->SetVertZPos(mPrimVertZ[0], mPrimVertZ[1]);
				mBasicEventCut->SetVzVpdVzDiff(mPrimVertVpdVz[0],mPrimVertVpdVz[1]);
				mBasicEventCut->SetVertRPos(mPrimVertR);
				mBasicEventCut->SetTofVsRef(mKRange[0], mKRange[1], mBRange[0], mBRange[1]);
				mBasicEventCut->CheckBadRuns(false);

				//
				// Set basic kaon single-particle cut
				//
				gregsProtonTrackCut *mBasicTrackCut = new gregsProtonTrackCut();
				mBasicTrackCut->SetCharge(mPosCharge);
				mBasicTrackCut->SetType(mTrackType);
				mBasicTrackCut->SetNHits(mTrackNHits[0], mTrackNHits[1]);
				mBasicTrackCut->SetNHitsFit(mTrackNHitsFit[0], mTrackNHitsFit[1]);
				mBasicTrackCut->SetAntiSplit(mTrackASplitVal);
				mBasicTrackCut->SetEta(mTrackEta[0], mTrackEta[1]);
				mBasicTrackCut->SetDCA(mTrackDCA[0], mTrackDCA[1]);
				mBasicTrackCut->SetDCAGlobal(mTrackDCA[0], mTrackDCA[1]);
				mBasicTrackCut->SetP(mTrackMom[0], mTrackMom[1]);

				mBasicTrackCut->SetDetectorSelection(mDetectorSelection);

				mBasicTrackCut->SetTpcMomentum(mTrackTpcMom[0], mTrackTpcMom[1]);
				mBasicTrackCut->SetTpcNSigmaElectron(mnSigmaElectron[0],mnSigmaElectron[1]);
				mBasicTrackCut->SetTpcNSigmaPion(mnSigmaPion[0], mnSigmaPion[1]);
				mBasicTrackCut->SetTpcNSigmaKaon(mnSigmaKaon[0], mnSigmaKaon[1]);
				mBasicTrackCut->SetTpcNSigmaProton(mnSigmaProton[0], mnSigmaProton[1]);

				mBasicTrackCut->SetTofMassSqr(mProtonTofMsqr[0],mProtonTofMsqr[1]);
				mBasicTrackCut->SetTofMomentum(mTrackTofMom[0], mTrackTofMom[1]);

				mBasicTrackCut->SetTnTMomentum(mTrackTnTMom[0], mTrackTnTMom[1]);
				mBasicTrackCut->SetTnTNSigmaProton(mTnTNSigmaProton[0], mTnTNSigmaProton[1]);

				//
				// Set basic kaon pair cut
				//
				gregsTrackPairCut *mBasicPairCut = new gregsTrackPairCut();
				mBasicPairCut->SetFracOfMergedRow(mFmrCut[0],mFmrCut[1]);
				mBasicPairCut->SetQuality(mSplitLevelCut[0],mSplitLevelCut[1]);
				mBasicPairCut->SetKt(mPairKt[0],mPairKt[1]);
				mBasicPairCut->SetQinv(mPairQinv[0],mPairQinv[1]);
				mBasicPairCut->SetAverageSeparation(mAveSep[0],mAveSep[1]);

				Int_t mLocalCharge, mMultBins, mMultLow, mMultHi, mEvents2Mix, mCurrentPid;

				//
				// StHbtAnalysis, cut, CF and monitor declarations
				//
				StHbtVertexMultSphAnalysis *mProtonMultAnalysis[CHARGE_BINS][SPH_BINS];

				//
				// Cuts
				//
				gregsEventCut                 *mProtonEventCut[CHARGE_BINS][SPH_BINS];
				gregsProtonTrackCut             *mProtonTrackCut[CHARGE_BINS][SPH_BINS];
				gregsTrackPairCut             *mProtonPairCut[CHARGE_BINS][SPH_BINS];

				//
				// CorrFctns
				//
				QinvCorrFctnKt_th *mProtonQinvMixKtCF[CHARGE_BINS][SPH_BINS];

#ifdef MONITORS
				//
				// Cut monitors
				//
				fxtEventCutMonitor *mFxtProtonEventCutMonitorPass[CHARGE_BINS][SPH_BINS];
				fxtTrackCutMonitor *mFxtProtonTrackCutMonitorPass[CHARGE_BINS][SPH_BINS];
				fxtPairCutMonitor  *mFxtProtonPairCutMonitorPass[CHARGE_BINS][SPH_BINS];
				fxtEventCutMonitor *mFxtProtonEventCutMonitorFail[CHARGE_BINS][SPH_BINS];
				fxtTrackCutMonitor *mFxtProtonTrackCutMonitorFail[CHARGE_BINS][SPH_BINS];
				fxtPairCutMonitor  *mFxtProtonPairCutMonitorFail[CHARGE_BINS][SPH_BINS];
#endif

				//
				// Analysis creation
				//
				float sphLo, sphHi;
				mMultLow = 0;
				mMultHi = 50;
				mMultBins = 10;
				mEvents2Mix = 10;
				for (Int_t iSph = 0; iSph < SPH_BINS; iSph++)
				{
								//
								// sphLo < sphericity <= sphHi
								//
								switch (iSph)
								{
								case 0:  sphLo = -0.1;     sphHi = 0.1;  break;
								default: sphLo = iSph*0.1; sphHi = (iSph + 1)*0.1;
								}

								for(Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
								{
												switch(iCharge)
												{
												case 0:   mLocalCharge = mPosCharge;    break;
												default:  mLocalCharge = mNegCharge;    break;
												};

												//
												// Create event cut
												//
												mProtonEventCut[iCharge][iSph] = new gregsEventCut(*mBasicEventCut);
												mProtonEventCut[iCharge][iSph]->SetEventMult(mMultLow, mMultHi);
												mProtonEventCut[iCharge][iSph]->SetSphericityCut(sphLo, sphHi);
#ifdef MONITORS
												mFxtProtonEventCutMonitorPass[iCharge][iSph] =
																new fxtEventCutMonitor("eventProtonPass", Form("_%d_%d", iCharge, iSph));
												mFxtProtonEventCutMonitorFail[iCharge][iSph] =
																new fxtEventCutMonitor("eventProtonFail", Form("_%d_%d", iCharge, iSph));
												mProtonEventCut[iCharge][iSph]->AddCutMonitor(mFxtProtonEventCutMonitorPass[iCharge][iSph],
																																			mFxtProtonEventCutMonitorFail[iCharge][iSph]);
#endif

												//
												//  Like-sign kaon analysis  
												//
												// create analysis
												mProtonMultAnalysis[iCharge][iSph] = new StHbtVertexMultSphAnalysis(mPrimVertZbins,
																																														mPrimVertZ[0],
																																														mPrimVertZ[1],
																																														mMultBins,
																																														mMultLow,
																																														mMultHi,
																																														10 /* sph bins */,
																																														0. /* sph lo   */,
																																														1. /* sph hi   */);

												mProtonMultAnalysis[iCharge][iSph]->SetEventCut(mProtonEventCut[iCharge][iSph]);

												//
												// Create single-track cuts and add them to the analysis
												//
												mProtonTrackCut[iCharge][iSph] = new gregsProtonTrackCut(*mBasicTrackCut);
												mProtonTrackCut[iCharge][iSph]->SetCharge(mLocalCharge);
#ifdef MONITORS
												mFxtProtonTrackCutMonitorPass[iCharge][iSph] =
																new fxtTrackCutMonitor(Form("_trackProtonPass_%d_%d_", iCharge, iSph), mChargedProtonMass);
												mFxtProtonTrackCutMonitorFail[iCharge][iSph] =
																new fxtTrackCutMonitor(Form("_trackProtonFail_%d_%d_", iCharge, iSph), mChargedProtonMass);
												mProtonTrackCut[iCharge][iSph]->AddCutMonitor(mFxtProtonTrackCutMonitorPass[iCharge][iSph],
																																			mFxtProtonTrackCutMonitorFail[iCharge][iSph]);
#endif
												mProtonMultAnalysis[iCharge][iSph]->SetFirstParticleCut(mProtonTrackCut[iCharge][iSph]);
												mProtonMultAnalysis[iCharge][iSph]->SetSecondParticleCut(mProtonTrackCut[iCharge][iSph]);

												//
												// Create pair cut
												//
												mProtonPairCut[iCharge][iSph] = new gregsTrackPairCut(*mBasicPairCut);
#ifdef MONITORS
												mFxtProtonPairCutMonitorPass[iCharge][iSph] =
																new fxtPairCutMonitor(Form("_pairProtonPass_%d_%d_", iCharge, iSph));
												mFxtProtonPairCutMonitorPass[iCharge][iSph]->SetParticleMass(mChargedProtonMass);
												mFxtProtonPairCutMonitorFail[iCharge][iSph] =
																new fxtPairCutMonitor(Form("_pairProtonFail_%d_%d_", iCharge, iSph));
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->SetParticleMass(mChargedProtonMass);
												mProtonPairCut[iCharge][iSph]->AddCutMonitor(mFxtProtonPairCutMonitorPass[iCharge][iSph],
																																		 mFxtProtonPairCutMonitorFail[iCharge][iSph]);
#endif
												mProtonMultAnalysis[iCharge][iSph]->SetPairCut(mProtonPairCut[iCharge][iSph]);

												//
												// Correlation function initialization
												//
												mProtonQinvMixKtCF[iCharge][iSph] =
																new QinvCorrFctnKt_th(Form("hProtonQinvMixKt_%d_%d", iCharge, iSph),
																											mQinvBins, mQinvVal[0], mQinvVal[1],
																											mPairKtBins, mPairKtVal[0], mPairKtVal[1]);

												mProtonMultAnalysis[iCharge][iSph]->AddCorrFctn(mProtonQinvMixKtCF[iCharge][iSph]);

												mProtonMultAnalysis[iCharge][iSph]->SetNumEventsToMix(mEvents2Mix); // set number of events to mix
												mHbtManager->AddAnalysis(mProtonMultAnalysis[iCharge][iSph]); // add analysis to the manager
								}
				}

				mChain->Init(); // StChain initialization

				//
				// Start processing
				//
				Int_t iRet = 0;
				Int_t mNEventsProcessed = 0;
				Float_t mPercentCounter = 0.05;
				Float_t mProgress = 0.;

				unsigned int mSubCounter = 1;              // time
				time_t mStartTime, mStopTime, mDiffTime;   // time
				float mFrac;                               // time
				mStartTime = time(0);                      // time

				// mNEvents = 15;
				std::cout << "Total Events: " << mNEvents << std::endl;
				for(Int_t iEvent=0; iEvent < mNEvents; iEvent++)
				{
								mProgress = (Float_t)iEvent/(Float_t)mNEvents;
								mNEventsProcessed++;

								if(mProgress >= mPercentCounter)
								{
												mPercentCounter += 0.05;
												mStopTime = time(0);
												mDiffTime = difftime(mStopTime, mStartTime);
												mFrac = (float)mDiffTime*(float)(mNEvents - iEvent)/(float)iEvent;
												mSubCounter++;
												std::cout << Form("Processing progress: %4.2f%%. Time left: %.1f sec", mProgress*100., mFrac)
																	<< std::endl;
								}

								mChain->Clear();
								iRet = mChain->Make(iEvent);
								if(iRet!=0)
								{
												std::cout << "[ERROR] - iRet: " << iRet << std::endl;
												break;
								}
				}

				mChain->Finish();

				TFile *oFile = new TFile(oFileName, "RECREATE"); // create output file

				//
				// Write hbt histograms to the file
				//
				std::cout << "Writing histograms to the output file...";

#ifdef MONITORS
				for(Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
				{
								for (Int_t iSph = 0; iSph < SPH_BINS; iSph++)
								{
												mFxtProtonEventCutMonitorPass[iCharge][iSph]->VertexZ()->Write();
												mFxtProtonEventCutMonitorPass[iCharge][iSph]->VertexYvsVertexX()->Write();
												mFxtProtonEventCutMonitorPass[iCharge][iSph]->RefMult()->Write();
												mFxtProtonEventCutMonitorPass[iCharge][iSph]->VpdVzDiff()->Write();
												mFxtProtonEventCutMonitorFail[iCharge][iSph]->VertexZ()->Write();
												mFxtProtonEventCutMonitorFail[iCharge][iSph]->VertexYvsVertexX()->Write();
												mFxtProtonEventCutMonitorFail[iCharge][iSph]->RefMult()->Write();
												mFxtProtonEventCutMonitorFail[iCharge][iSph]->VpdVzDiff()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->DCAGlobal()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->Nhits()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->P()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->Pt()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PtVsNsigmaPion()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PtVsNsigmaKaon()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PtVsNsigmaProton()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PvsDedx()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->Rapidity()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PseudoRapidity()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PvsMassSqr()->Write();
												mFxtProtonTrackCutMonitorPass[iCharge][iSph]->PvsInvBeta()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->DCAGlobal()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->Nhits()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->P()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->Pt()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PtVsNsigmaPion()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PtVsNsigmaKaon()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PtVsNsigmaProton()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PvsDedx()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->Rapidity()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PseudoRapidity()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PvsMassSqr()->Write();
												mFxtProtonTrackCutMonitorFail[iCharge][iSph]->PvsInvBeta()->Write();

												mFxtProtonPairCutMonitorPass[iCharge][iSph]->Kt()->Write();
												mFxtProtonPairCutMonitorPass[iCharge][iSph]->Pt1Pt2Qinv()->Write();
												mFxtProtonPairCutMonitorPass[iCharge][iSph]->FMRvsQinv()->Write();
												mFxtProtonPairCutMonitorPass[iCharge][iSph]->SLvsQinv()->Write();
												mFxtProtonPairCutMonitorPass[iCharge][iSph]->AveSepVsQinv()->Write();
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->Kt()->Write();
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->Pt1Pt2Qinv()->Write();
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->FMRvsQinv()->Write();
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->SLvsQinv()->Write();
												mFxtProtonPairCutMonitorFail[iCharge][iSph]->AveSepVsQinv()->Write();
								}
				}

#endif

				for (Int_t iSph = 0; iSph < SPH_BINS; iSph++)
				{
								for (Int_t iCharge = 0; iCharge < CHARGE_BINS; iCharge++)
								{
												for (Int_t iKt = 0; iKt < mPairKtBins; iKt++)
												{
																mProtonQinvMixKtCF[iCharge][iSph]->Numerator(iKt)->Write();
																mProtonQinvMixKtCF[iCharge][iSph]->Denominator(iKt)->Write();
												}
								}
				}

				std::cout << "\t[DONE]" << std::endl;

				//
				// Write the output file and close it
				//
				std::cout << "Saving the output file...";
				oFile->Close();
				std::cout << "\t[DONE]" << std::endl;

				delete mChain;

				std::cout << "********************************" << std::endl
									<< "*       simProtonHBT_pp200_1D  *" << std::endl
									<< "*              Finish          *" << std::endl
									<< "********************************" << std::endl << std::endl;
}
