#include "TROOT.h"
#include "TSystem.h"
#include "TChain.h"
#include "TFile.h"
#include "TString.h"
#include <iostream>

const Char_t *defaultInFile = "test.list";
const Char_t *defaultOutFile = "oMuDstQA.root";


void MuDst_pp200_y2015_QA(const Char_t *inFileList = defaultInFile,
                          const Char_t *outFileName = defaultOutFile)
{
  std::cout << "**************************************" << std::endl
      << "*                                    *" << std::endl
      << "*  Hello, this is quality assurance  *" << std::endl
      << "*   for the MuDst                    *" << std::endl
      << "*                                    *" << std::endl
      << "**************************************" << std::endl;
  //
  // Load libraries
  //
  std::cout << "Loading libraries..." << std::endl;
  gROOT->LoadMacro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
  loadSharedLibraries();
  gSystem->Load("libMinuit");
  gSystem->Load("StMuDSTMaker");
  gSystem->Load("StChain");
  gSystem->Load("StMuDstQAMaker");
  std::cout << "Libraries have been successfully loaded" << std::endl;

  //
  // Create chain
  //
  StChain *chain = new StChain("StChain");
  chain->SetDebug(0);
  StMuDebug::setLevel(0);

  //
  // Create MuDst reader
  //
  int nMaxFileToRead = 1e9;
  std::cout << Form("Try to read file: %s", inFileList) << std::endl;
  StMuDstMaker *muDstMaker = new StMuDstMaker(0, 0, "", inFileList, "MuDst", nMaxFileToRead);
  muDstMaker->SetStatus("*",0);
  muDstMaker->SetStatus("Event*",1);
  muDstMaker->SetStatus("MuEvent*",1);
  muDstMaker->SetStatus("PrimaryVertices*",1);
  muDstMaker->SetStatus("PrimaryTracks*",1);
  muDstMaker->SetStatus("GlobalTracks*",1);
  muDstMaker->SetStatus("BTofHeader*",1);
  TChain *currentTree = muDstMaker->chain();
  Long64_t nEvents = currentTree->GetEntries();
  std::cout << Form("Number of events in TTree: %d", nEvents) << std::endl;

  //
  // Create StMuDstQAMaker
  //
  StMuDstQAMaker *QAmaker = new StMuDstQAMaker(muDstMaker, outFileName);
  QAmaker->SetTriggerCut(false);
  QAmaker->SetTracksLoop(false);
  QAmaker->SetVtxZCut(-70., 70.);
  QAmaker->SetVtxRCut(0., 3.);
  QAmaker->SetVtxShift(0., 0.);
  QAmaker->SetVtxVpdVzDiffCut(-7., 7.);

  QAmaker->SetTriggerId(480006); // trigger selection for pp200 y2015 P16id VPDMB
  QAmaker->SetTriggerId(470001);
  QAmaker->SetTriggerId(470004);
  QAmaker->SetTriggerId(470007);
  QAmaker->SetTriggerId(470014);
  QAmaker->SetTriggerId(470017);
  QAmaker->SetTriggerId(470904);
  QAmaker->SetTriggerId(470914);
  QAmaker->SetTriggerId(490004);
  QAmaker->SetTriggerId(490007);
  QAmaker->SetTriggerId(490904);
  QAmaker->SetTriggerId(470401); // JP2
  QAmaker->SetTriggerId(470402); // JP2-bsmd
  QAmaker->SetTriggerId(470404); // JP1
  QAmaker->SetTriggerId(470201); // BHT0*VPDMB-30
  QAmaker->SetTriggerId(470202); // BHT1*VPDMB-30
  QAmaker->SetTriggerId(470206); // BHT1*VPDMB-30-nobsmd
  QAmaker->SetTriggerId(470211); // BHT0*VPDMB-5
  QAmaker->SetTriggerId(480004); // VPDMB-novtx
  QAmaker->SetTriggerId(480007); // VPDMB-5-trgonly
  QAmaker->SetTriggerId(480904); // VPDMB-30
  QAmaker->SetTriggerId(480401); // JP2
  QAmaker->SetTriggerId(480404); // JP1
  QAmaker->SetTriggerId(480411); // JP2
  QAmaker->SetTriggerId(480414); // JP1
  QAmaker->SetTriggerId(480201); // BHT0*VPDMB-5
  QAmaker->SetTriggerId(480202); // BHT1*VPDMB-30

  chain->Init();
  Int_t iReturn = 0;
  Int_t nEventsProcessed = 0;
  Float_t percentCounter = 0.01;
  Float_t progress = 0.;

  for (Int_t iEvent = 0; iEvent < nEvents; iEvent++) {
    progress = (Float_t)iEvent/(Float_t)nEvents;
    nEventsProcessed++;

    if(progress >= percentCounter) {
      percentCounter += 0.01;
      std::cout << Form("Processing progress: %4.2f%%", percentCounter*100.)
          << std::endl;
    }

    chain->Clear();
    iReturn = chain->Make(iEvent);
    if (iReturn != 0) {
      std::cout << "Error has been occured. Event processing has been stopped"
          << std::endl;
      break;
    }
  }

  chain->Finish();

  delete chain;

  std::cout << "*************************" << std::endl
      << "*        Finish         *" << std::endl
      << "*************************" << std::endl;
}
