//
// This macro based on code: $STAR/StRoot/StarGenerator/macros/starsim.pythia8.C
// Author: Nikita Ermakov (coffe92@gmail.com)
//
#include <TChain.h>
#include <stdio.h>

void EmcicStarGen(const char *inList = "in.list", const float etaMax = 1.0)
{
  std::cout << "Loading libraries..." << std::endl;
  gROOT->LoadMacro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
  loadSharedLibraries();
  gSystem->Load("libMinuit");
  gSystem->Load("StarClassLibrary");
  gSystem->Load("libgsl");
  gSystem->Load("libgslcblas");
  gSystem->Load("libVMC.so");
  gSystem->Load("libStDb_Tables.so");
  gSystem->Load("libgen_Tables.so");
  gSystem->Load("libgeometry_Tables.so");
  gSystem->Load("libsim_Tables.so");
  gSystem->Load("libStarMagField.so");
  gSystem->Load("libSt_db_Maker.so");
  gSystem->Load("libSt_g2t.so");
  gSystem->Load("libSt_geant_Maker.so");
  gSystem->Load("StarGeneratorUtil.so");
  gSystem->Load("StarGeneratorEvent.so");
  gSystem->Load("StarGeneratorBase.so");  
  gSystem->Load("Pythia8_1_62.so");
  gSystem->Load("libMathMore.so");
  gSystem->Load("Pythia8_1_62.so");

  std::cout << "Libraries have been successfully loaded" << std::endl;

  TChain *chain = new TChain("genevents", "genevents");
  StarGenEvent *mSGEvent = new StarGenEvent;

  FILE *inFile = fopen(inList, "r");
  while (1)
  {
    char buf[255];
    fgets(buf, 255, inFile);
    buf[strlen(buf) - 1] = 0;
    if (feof(inFile))   break;
    else                chain->Add(buf);
  }
  chain->SetBranchAddress("primaryEvent", &mSGEvent);

  unsigned long Nevents = chain->GetEntries();
  TDatabasePDG *db = TDatabasePDG::Instance();
  // charged + uncharged tracks
  double totalTracks = 0.;
  double totalE = 0.;
  double totalPtSqr = 0.;
  double totalPzSqr = 0.;
  double totalESqr = 0.;
  // only charged tracks
  double totalTracksCh = 0.;
  double totalECh = 0.;
  double totalPtSqrCh = 0.;
  double totalPzSqrCh = 0.;
  double totalESqrCh = 0.;

  for (unsigned long iEvent = 0; iEvent < Nevents; iEvent++)
  {
    chain->GetEntry(iEvent);
    if (!mSGEvent)
    {
      printf("[ERROR]: mSGEvent == NULL!\n");
      continue;
    }

    int nTracks = mSGEvent->GetNumberOfParticles(); // number of tracks in an event
    if (iEvent % 100)   printf("event %d of %d (%d tracks)\n", iEvent, Nevents, nTracks);
    totalTracks += nTracks;
    for (int iTrack = 0; iTrack < nTracks; iTrack++)
    {
      StarGenParticle      *track   = (*mSGEvent)[iTrack];
      if (!track)
      {
        printf("[ERROR]: track == NULL!\n");
        continue;
      }
      const TLorentzVector &mom     = track->momentum();

      // check pseudorapidity
      if (etaMax != 0. && (mom.Pt() <= 1e-10 || fabs(mom.PseudoRapidity()) > etaMax))   continue;

      TParticlePDG         *partPdg = db->GetParticle(track->GetId());
      if (!partPdg)
      {
        printf("[ERROR]: partPdg == NULL! pdg = %d\n", track->GetId());
        continue;
      }

      char charge = partPdg->Charge();

      double E = mom.E();
      double ptSqr = mom.Perp2();
      double pzSqr = mom.Pz()*mom.Pz();

      totalE += E;
      totalESqr += E*E;
      totalPtSqr += ptSqr;
      totalPzSqr += pzSqr;

      if (charge != 0)
      {
        totalECh += E;
        totalESqrCh += E*E;
        totalPtSqrCh += ptSqr;
        totalPzSqrCh += pzSqr;
        totalTracksCh++;
      }
    }
  }

  // charged + uncharged tracks
  double meanE = totalE/totalTracks;
  double meanPtSqr = totalPtSqr/totalTracks;
  double meanPzSqr = totalPzSqr/totalTracks;
  double meanESqr = totalESqr/totalTracks;
  double sqrMeanE = meanE*meanE;

  // only charged tracks
  double meanECh = totalECh/totalTracksCh;
  double meanPtSqrCh = totalPtSqrCh/totalTracksCh;
  double meanPzSqrCh = totalPzSqrCh/totalTracksCh;
  double meanESqrCh = totalESqrCh/totalTracksCh;
  double sqrMeanECh = meanE*meanE;

  // output
  printf("--- ALL TRACKS ---\n\t<E> = %e [GeV]   <E>^{2} = %e [GeV^{2}]   <E^{2}> = %e [GeV^{2}]\n\t"
         "<p_{t}^{2}> = %e [(GeV/c)^{2}]   <p_{z}^{2}> = %e [(GeV/c)^{2}]\n\ttotal tracks = %e   <N> = %e\n\n",
         meanE, sqrMeanE, meanESqr, meanPtSqr, meanPzSqr, totalTracks, totalTracks/(double)Nevents);
  printf("--- ONLY CHARGED TRACKS ---\n\t<E> = %e [GeV]   <E>^{2} = %e [GeV^{2}]   <E^{2}> = %e [GeV^{2}]\n\t"
         "<p_{t}^{2}> = %e [(GeV/c)^{2}]   <p_{z}^{2}> = %e [(GeV/c)^{2}]\n\ttotal tacks = %e   <N> = %e\n",
         meanECh, sqrMeanECh, meanESqrCh, meanPtSqrCh, meanPzSqrCh, totalTracksCh, totalTracksCh/(double)Nevents);
}
