#include <TFile.h>
#include "TROOT.h"
#include "TSystem.h"
#include "TChain.h"
#include "TTree.h"



void sphericity(const char *flist = "test.list",
                const char *ofile = "sphericity.root")
{
    std::cout << "Input file list = " << flist << '\n';
    if (!flist) {
        std::cout << "Input file list = NULL!\n";
        return;
    }

    //
    // Load libraries
    //
    gROOT->LoadMacro("$STAR/StRoot/StMuDSTMaker/COMMON/macros/loadSharedLibraries.C");
    loadSharedLibraries();
    gSystem->Load("StMuDSTMaker");
    gSystem->Load("StChain");
    gSystem->Load("StSphericity");

    StChain      *chain = 0;
    StSphericity *maker = 0;

    //
    // List of member links in the chain
    //
    chain = new StChain("StChain");
    chain->SetDebug(0);
    StMuDebug::setLevel(0);

    //
    // Create MuDst reader
    //
    int nMaxFileToRead = 1e9;
    std::cout << Form("Try to read file: %s", flist) << std::endl;
    StMuDstMaker *muDstMaker = new StMuDstMaker(0, 0, "", flist, "MuDst", 1e9);
    muDstMaker->SetStatus("*",0);
    muDstMaker->SetStatus("Event*",1);
    muDstMaker->SetStatus("MuEvent*",1);
    muDstMaker->SetStatus("PrimaryVertices*",1);
    muDstMaker->SetStatus("PrimaryTracks*",1);
    muDstMaker->SetStatus("GlobalTracks*",1);
    muDstMaker->SetStatus("BTofHeader*",1);
    TChain *currentTree = muDstMaker->chain();
    Long64_t nEvents = currentTree->GetEntries();
    std::cout << Form("Number of events in TTree: %d", nEvents) << std::endl;

    maker = new StSphericity(muDstMaker, ofile);
    /*
    maker->AddTrigId(470001); // vpdmb -nobsmd
    maker->AddTrigId(470004); // vpdmb
    maker->AddTrigId(470007);
    maker->AddTrigId(470014);
    maker->AddTrigId(470017);
    maker->AddTrigId(470904);
    maker->AddTrigId(490004);
    maker->AddTrigId(490007);
    maker->AddTrigId(490904);
    */
//    maker->AddTrigId(370022); // bbcmb
//    maker->AddTrigId(370501); // bht0 -vpdmb
//    maker->AddTrigId(370511); // bht1 -vpdmb
//    maker->AddTrigId(370531); // bht2
//    maker->AddTrigId(370542); // bht0 -bbcmb -tof0
//    maker->AddTrigId(370546); // bht1 -bbcmb -tof0
//    maker->AddTrigId(370522); // bht2 -bbcmb
//    maker->AddTrigId(370601); // jp0
//    maker->AddTrigId(370611); // jp1
//    maker->AddTrigId(370621); // jp2
//    maker->AddTrigId(370641); // ajp
//    maker->AddTrigId(370301); // bbcmb -tof0
//    maker->AddTrigId(370361); // tofmult3 -vpd
//    maker->AddTrigId(370341); // tofmult4
    chain->Init();

    //
    // Loop over the links in the chain
    //
    int iret = 0;
    std::cout << "Entries in chain = " << nEvents << std::endl;

    for (int iev = 0; iev < nEvents; iev++)
    {
        chain->Clear();
        iret = chain->Make(iev);
        if (iret)
        {
            std::cout << "Error in Make occured. Error code: " << iret << std::endl;
            std::cout << "iev = " << iev << '\n';
            break;
        }
    }
    chain->Finish();
    //Cleanup
    delete maker;
    delete chain;
}
