#include "HistoCollector.h"

//--------------------------------------------------------------------------
HistoCollector::HistoCollector()
{
  mFile = 0;
  mQinvBins = 30;
  mQinvLo = 0.0;
  mQinvHi = 0.3;

  mHistName = "hKaonQinvMixKt";
  mHistTmpName = "hKaon1DKt";
  mNoChargeDep = false;

  mKtBins   = 11;
  mSphBins  = 10;
  mMultBins = 10;
}

//--------------------------------------------------------------------------
HistoCollector::~HistoCollector()
{
  if (mFile)
  {
    mFile->Close();
    delete mFile;
  }
}

//--------------------------------------------------------------------------
void HistoCollector::OpenFile()
{
  cout << "Opening file: " << mFileName << "... ";
  mFile = new TFile(mFileName, "READ");
  if (mFile)
    cout << "[DONE]\n";
  else
    cout << "[ERROR] file does not exist\n";
}

//--------------------------------------------------------------------------
void HistoCollector::LoadData()
{
  OpenFile();
  ReadHistograms();
}

//--------------------------------------------------------------------------
void HistoCollector::ReadHistograms()
{
  cout << "Retrieving histograms from file... ";

  //
  // 1D histograms
  //
  TH1F *hNumTemp1D = NULL;
  TH1F *hDenTemp1D = NULL;

  if (!mNoChargeDep) // for unlike sign
  {
      for (int iCharge = 0; iCharge < 2; iCharge++)
      {
          for (int iMult = 0; iMult < mMultBins; iMult++)
          {
              for (int iSph = 0; iSph < mSphBins; iSph++)
              {
                  for (int iKt = 0; iKt < mKtBins; iKt++)
                  {
                      //
                      // Numerator
                      //
                      hNumTemp1D = (TH1F *)mFile->Get(Form("%s_%d_%d_%d_Num_bin_%d",
                                                           mHistName, iCharge, iMult, iSph, iKt));
                      hNumTemp1D->SetName(Form("%sNum_%d_%d_%d_%d",
                                               mHistTmpName, iCharge, iMult, iSph, iKt));

                      //
                      // Denominator
                      //
                      hDenTemp1D = (TH1F *)mFile->Get(Form("%s_%d_%d_%d_Den_bin_%d",
                                                           mHistName, iCharge, iMult, iSph, iKt));
                      hDenTemp1D->SetName(Form("%sDen_%d_%d_%d_%d",
                                               mHistTmpName, iCharge, iMult, iSph, iKt));

                      //
                      // Add histograms to the vectors
                      //
                      h1DNum.push_back(hNumTemp1D);
                      h1DDen.push_back(hDenTemp1D);
                  }
              }
          }
      }
  }
  else
  {
      for (int iKt = 0; iKt < mKtBins; iKt++)
      {
          //
          // Numerator
          //
          hNumTemp1D = (TH1F *)mFile->Get(Form("%s_Num_bin_%d",
                                               mHistName, iKt));
          hNumTemp1D->SetName(Form("%sNum_%d",
                                   mHistTmpName, iKt));

          //
          // Denominator
          //
          hDenTemp1D = (TH1F *)mFile->Get(Form("%s_Den_bin_%d",
                                               mHistName, iKt));
          hDenTemp1D->SetName(Form("%sDen_%d",
                                   mHistTmpName, iKt));

          //
          // Add histograms to the vectors
          //
          h1DNum.push_back(hNumTemp1D);
          h1DDen.push_back(hDenTemp1D);
      }
  }

  cout << "[DONE]\n";
  cout << Form("Number of histos in 1D numerator:   %d\n", h1DNum.size())
       << Form("Number of histos in 1D denominator: %d\n", h1DDen.size());
}

//--------------------------------------------------------------------------
TH1F* HistoCollector::BuildKt1DNum(int charge, int binMultLo, int binMultHi,
                                   int binSphLo, int binSphHi,
                                   int binKtLo, int binKtHi)
{
    TH1F *hNumSum = 0;

    if (!mNoChargeDep)
    {
        TString sName = "h";
        sName += "Num1DKt_";
        sName += charge;
        sName += "_";
        sName += binMultLo;
        sName += "_";
        sName += binMultHi;
        sName += "_";
        sName += binSphLo;
        sName += "_";
        sName += binSphHi;
        sName += "_";
        sName += binKtLo;
        sName += "_";
        sName += binKtHi;

        hNumSum = new TH1F(sName.Data(), sName.Data(),
                           mQinvBins, mQinvLo, mQinvHi);
        hNumSum->Sumw2();

        if (charge < 2) // positive or negative particles
        {
            for (int iMult = binMultLo; iMult <= binMultHi; iMult++)
            {
                for (int iSph = binSphLo; iSph <= binSphHi; iSph++)
                {
                    for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
                    {
                        hNumSum->Add(h1DNum[charge*mMultBins*mSphBins*mKtBins + iMult*mSphBins*mKtBins + iSph*mKtBins + iKt]);
                    }
                }
            }
        }
        else // sum of charges
        {
            for (int iCharge = 0; iCharge < 2; iCharge++)
            {
                for (int iMult = binMultLo; iMult <= binMultHi; iMult++)
                {
                    for (int iSph = binSphLo; iSph <= binSphHi; iSph++)
                    {
                        for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
                        {
                            hNumSum->Add(h1DNum[iCharge*mMultBins*mSphBins*mKtBins + iMult*mSphBins*mKtBins + iSph*mKtBins + iKt]);
                        }
                    }
                }
            }
        }
    }
    else
    {
        TString sName = "h";
        sName += "Num1DKt_";
        sName += binKtLo;
        sName += "_";
        sName += binKtHi;

        hNumSum = new TH1F(sName.Data(), sName.Data(),
                           mQinvBins, mQinvLo, mQinvHi);
        hNumSum->Sumw2();

        for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
        {
            hNumSum->Add(h1DNum[iKt]);
        }
    }

    return hNumSum;
}

//--------------------------------------------------------------------------
TH1F* HistoCollector::BuildKt1DDen(int charge, int binMultLo, int binMultHi,
                                   int binSphLo, int binSphHi,
                                   int binKtLo, int binKtHi)
{
    TH1F *hDenSum = 0;

    if (!mNoChargeDep)
    {
        TString sName = "h";
        sName += "Den1DKt_";
        sName += charge;
        sName += "_";
        sName += binMultLo;
        sName += "_";
        sName += binMultHi;
        sName += "_";
        sName += binSphLo;
        sName += "_";
        sName += binSphHi;
        sName += "_";
        sName += binKtLo;
        sName += "_";
        sName += binKtHi;

        hDenSum = new TH1F(sName.Data(), sName.Data(),
                           mQinvBins, mQinvLo, mQinvHi);
        hDenSum->Sumw2();

        if(charge < 2) // positive or negative particles
        {
            for (int iMult = binMultLo; iMult <= binMultHi; iMult++)
            {
                for (int iSph = binSphLo; iSph <= binSphHi; iSph++)
                {
                    for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
                    {
                        hDenSum->Add(h1DDen[charge*mMultBins*mSphBins*mKtBins + iMult*mSphBins*mKtBins + iSph*mKtBins + iKt]);
                    }
                }
            }
        }
        else // sum of charges
        {
            for (int iCharge = 0; iCharge < 2; iCharge++)
            {
                for (int iMult = binMultLo; iMult <= binMultHi; iMult++)
                {
                    for (int iSph = binSphLo; iSph <= binSphHi; iSph++)
                    {
                        for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
                        {
                            hDenSum->Add(h1DDen[iCharge*mMultBins*mSphBins*mKtBins + iMult*mSphBins*mKtBins + iSph*mKtBins + iKt]);
                        }
                    }
                }
            }
        }
    }
    else
    {
        TString sName = "h";
        sName += "Den1DKt_";
        sName += binKtLo;
        sName += "_";
        sName += binKtHi;

        hDenSum = new TH1F(sName.Data(), sName.Data(),
                           mQinvBins, mQinvLo, mQinvHi);
        hDenSum->Sumw2();

        for (int iKt = binKtLo; iKt <= binKtHi; iKt++)
        {
            hDenSum->Add(h1DDen[iKt]);
        }
    }

    return hDenSum;
}
