#ifndef HistoCollector_h
#define HistoCollector_h

#include <TFile.h>
#include <TSystem.h>
#include <TH3.h>
#include <TString.h>
#include <TAttMarker.h>
#include <iostream>
#include <vector>

using namespace std;



//--------------------------------------------------------------------------
class HistoCollector
{
 public:

  HistoCollector();
  ~HistoCollector();

  inline void SetQinvBins(int qInvBins, float qInvLo, float qInvHi);
  inline void SetSphBins(int bins);
  inline void SetMultBins(int bins);
  inline void SetKtBins(int bins);
  inline void SetFileName(const char *name);
  inline void SetHistName(const char *hname);
  inline void SetHistTmpName(const char *hname);
  inline void SetChargeDep(bool val) { mNoChargeDep = !val; }

  void LoadData();

  //
  // 1D histograms
  //
  TH1F *BuildKt1DNum(int charge, int binMultLo, int binMultHi,
                     int binSphLo, int binSphHi,
                     int binKtLo, int binKtHi);
  TH1F *BuildKt1DDen(int charge, int binMultLo, int binMultHi,
                     int binSphLo, int binSphHi,
                     int binKtLo, int binKtHi);

 private:

  void OpenFile();
  void ReadHistograms();
  TFile *mFile;

  int mKtBins;
  int mSphBins;
  int mMultBins;
  const char *mFileName;
  const char *mHistName;
  const char *mHistTmpName;
  int   mQinvBins;
  float mQinvLo;
  float mQinvHi;
  bool  mNoChargeDep;

  //
  // 1D histograms
  //
  std::vector<TH1F *> h1DNum;
  std::vector<TH1F *> h1DDen;
  std::vector<float>  scale;
};



//--------------------------------------------------------------------------
void HistoCollector::SetQinvBins(int qInvBins, float qInvLo, float qInvHi)
{
  mQinvBins = qInvBins;
  mQinvLo = qInvLo;
  mQinvHi = qInvHi;
}

//--------------------------------------------------------------------------
void HistoCollector::SetKtBins(int bins)
{
  mKtBins = bins;
}

//--------------------------------------------------------------------------
void HistoCollector::SetSphBins(int bins)
{
    mSphBins = bins;
}

//--------------------------------------------------------------------------
void HistoCollector::SetMultBins(int bins)
{
    mMultBins = bins;
}

//--------------------------------------------------------------------------
void HistoCollector::SetFileName(const char *name)
{
  mFileName = name;
}

//--------------------------------------------------------------------------
void HistoCollector::SetHistName(const char *hname)
{
    mHistName = hname;
}

//--------------------------------------------------------------------------
void HistoCollector::SetHistTmpName(const char *hname)
{
    mHistTmpName = hname;
}

#endif
