#include "HistoCollector.h"
#include "HbtFitter.h"
#include <TCanvas.h>
#include <TLegend.h>
#include <TMath.h>
#include <TROOT.h>
#include <TGraphErrors.h>
#include <vector>
#include <iostream>
#include <fstream>

using namespace std;

int qInvBins = 300;
double qInvLo = 0.;
double qInvHi = 3.;
int NKtBins = 22;
int SourceFunc = 0;   //Gauss
int NonFemtoFunc = 0; //Constant
double FitRangeLo = 0.0;
double FitRangeHi = 1.0;
double CoulombSR = 0.5;
const char *inFileName = "input/pp510_exp.root";
const char *inFileNameMc = "input/pp510_tune370.root";
const char *outFileName = "output/pp510_exp.root";
const char *outTextName = "output/pp510.txt";



HbtFitter *DoFit(TH1F* hNum, TH1F* hDen,
                 int aSourceFunc = 0, int aNonFemtoFunc = 0);
HbtFitter *DoFitMc(TH1F* hNum, TH1F* hDen, TH1F *hNumMc, TH1F *hDenMc,
                   HbtFitter *fitter, float coulRad);
void SetTGraphStyle(TGraphErrors *gr, int plot);
/*
void SetProjectionStyle(TH1* h, int proj, Bool_t isFit=false);
*/

//--------------------------------------------------------------------------
void Process_new(int mult = 0, bool coulSyst = false, bool fitRSyst = false)
{
  /*
  gSystem->Load("HistoCollector_cxx.so");
  gSystem->Load("HbtFitter_cxx.so");
  */
  double PionMassSqr = 0.019479835;
  double KaonMassSqr = 0.24371698;
  float coulRad[] = {1.0, 1.0, 1.0, 1.0};
  HistoCollector mHistCol, mHistColMc;

  //
  // Real data
  //
  mHistCol.SetFileName(inFileName);
  mHistCol.SetQinvBins(qInvBins, qInvLo, qInvHi);
  mHistCol.SetKtBins(NKtBins);
  mHistCol.LoadData();
  //
  // MC data
  //
  mHistColMc.SetFileName(inFileNameMc);
  mHistColMc.SetQinvBins(qInvBins, qInvLo, qInvHi);
  mHistColMc.SetKtBins(NKtBins);
  mHistColMc.LoadData();
  mHistColMc.SetHistTmpName("hKaonMc1DKt");

  int nKtSumBins = 4;
  int FitKtArray[4][2] = {{0, 4}, {5, 8}, {9, 11}, {12, 21}};
  std::vector<TH1F *> hNum;
  std::vector<TH1F *> hDen;
  std::vector<TH1F *> hNumMc;
  std::vector<TH1F *> hDenMc;
  std::vector<TH1F *> hRat;


  for (int iCharge = 0; iCharge < 3; iCharge++)
  {
    for (int iKtBin = 0; iKtBin < nKtSumBins; iKtBin++)
    {
        //
        // Real data
        //
        TH1F *hA = mHistCol.BuildKt1DNum(iCharge,
                                         FitKtArray[iKtBin][0],
                                         FitKtArray[iKtBin][1]);
        TH1F *hB = mHistCol.BuildKt1DDen(iCharge,
                                         FitKtArray[iKtBin][0],
                                         FitKtArray[iKtBin][1]);
        TH1F *hist = new TH1F(Form("hRat_%d_%d", iCharge, iKtBin),
                              ";C(q_{inv});q_{inv} (GeV/c)",
                              qInvBins, qInvLo, qInvHi);
        hist->Divide(hA, hB);
        hist->Write();

        hNum.push_back(hA);
        hDen.push_back(hB);
        //
        // MC data
        //
        hA = mHistColMc.BuildKt1DNum(iCharge,
                                     FitKtArray[iKtBin][0],
                                     FitKtArray[iKtBin][1]);
        hB = mHistColMc.BuildKt1DDen(iCharge,
                                     FitKtArray[iKtBin][0],
                                     FitKtArray[iKtBin][1]);

        hNumMc.push_back(hA);
        hDenMc.push_back(hB);
    }
  }

  HbtFitter *fitter = new HbtFitter(4, 0 /* gaus */, 2 /* pol1 */, false);
  TFile *oFile = new TFile(outFileName, "RECREATE");
  ofstream txtFile;
  txtFile.open(outTextName);
  oFile->cd();
  float lambda[4], R[4], chi2[4];
  float errLambda[4], errR[4], errChi2[4];
  float kt[4] = {0.15, 0.375, 0.55, 0.875};
  float errKt[4] = {0};
  for (int iKt = 0; iKt < nKtSumBins; iKt++)
  {
      fitter->Clear();
      DoFitMc(hNum.at(2*nKtSumBins + iKt), hDen.at(2*nKtSumBins + iKt),
              hNumMc.at(2*nKtSumBins + iKt), hDenMc.at(2*nKtSumBins + iKt),
              fitter, coulRad[iKt]);

      vector<double> femtoPar = fitter->GetFemto1DParameters();
      vector<double> nonFemtoPar = fitter->GetNonFemto1DParameters();
      vector<double> errFemtoPar = fitter->GetFemto1DParError();

      float ktLo = 0.05 + 0.05*FitKtArray[iKt][0];
      float ktHi = 0.05 + 0.05*FitKtArray[iKt][1];

      txtFile
          << "****** Kt bin = " << iKt << " ******\n"
          << "KtLo = " << ktLo << " [GeV/c]"
          << ", KtHi = " << ktHi << " [GeV/c]"
          << ", KtAvg = " << ktLo + (ktHi - ktLo)*0.5 << " [GeV/c]\n"
          << "Lambda = " << femtoPar[0]
          << ", R = " << femtoPar[1] << " [fm]"
          << ", N = " << nonFemtoPar[0]
          << ", a = " << nonFemtoPar[1]
          << ", b = " << nonFemtoPar[2] << "\n"
          << "Chi2: " << fitter->Get1dChi2() << " NDF: " << fitter->Get1dNDF()
          << " Chi2/NDF: " << fitter->Get1dChi2PerNDF() << "\n\n";

      lambda[iKt] = femtoPar[0];
      R[iKt] = femtoPar[1];
      chi2[iKt] = fitter->Get1dChi2PerNDF();
      errLambda[iKt] = errFemtoPar[0];
      errR[iKt] = errFemtoPar[1];
      errChi2[iKt] = 0.0; // little bollocks

      TH1F *hCF = fitter->Get1DCorrFctn();
      TH1F *hCFfit = fitter->Get1DCorrFctnFit();
      hCF->SetName(Form("hCF_%d", iKt));
      hCFfit->SetName(Form("hCFfit_%d", iKt));
      hCFfit->SetLineColor(kRed);
      hCF->Write();
      hCFfit->Write();
  }

  txtFile.close();

  //
  // Plot TGraph's
  //
  TGraphErrors *grLambda = new TGraphErrors(nKtSumBins, kt, lambda, errKt, errLambda);
  TGraphErrors *grR = new TGraphErrors(nKtSumBins, kt, R, errKt, errR);
  TGraphErrors *grChi2 = new TGraphErrors(nKtSumBins, kt, chi2, errKt, errChi2);

  SetTGraphStyle(grLambda, 0);
  SetTGraphStyle(grR, 1);
  SetTGraphStyle(grChi2, 2);
  grLambda->Write();
  grR->Write();
  grChi2->Write();

  oFile->Close();
  /*
     vector<double> LamVect, RoutVect, RsideVect, RlongVect, Chi2Vect;
     vector<double> LamErrVect, RoutErrVect, RsideErrVect, RlongErrVect;
     vector<double> retParVect;

  //HbtFitter *myFitter = NULL;
  double Chi2Arr[3][nKtSumBins];
  double LamArr[nKtSumBins];
  double RoutArr[nKtSumBins];
  double RsideArr[nKtSumBins];
  double RlongArr[nKtSumBins];
  double LamErrArr[nKtSumBins];
  double RoutErrArr[nKtSumBins];
  double RsideErrArr[nKtSumBins];
  double RlongErrArr[nKtSumBins];
  double RosRatArr[nKtSumBins];
  double RosRatErrArr[nKtSumBins];
  double RosDiffArr[nKtSumBins];
  double RosDiffErrArr[nKtSumBins];
  double RlongSqrArr[nKtSumBins];
  double RlongSqrErrArr[nKtSumBins];
  double KtArr[nKtSumBins];
  double KtErrArr[nKtSumBins];
  double MtArr[nKtSumBins];
  double KtShift = 0.1;
  double KtStep = 0.1;
  double KtLo,KtHi,KtVal,MtVal;
  double MassSqr = KaonMassSqr;

  //Kt dependence
  TGraphErrors *LamKtGrErr[3];
  TGraphErrors *RoutKtGrErr[3];
  TGraphErrors *RsideKtGrErr[3];
  TGraphErrors *RlongKtGrErr[3];
  TGraphErrors *RosRatKtGrErr[3];
  TGraphErrors *RosDiffKtGrErr[3];
  TGraphErrors *RlongSqrKtGrErr[3];

  //Mt dependence
  TGraphErrors *LamMtGrErr[3];
  TGraphErrors *RoutMtGrErr[3];
  TGraphErrors *RsideMtGrErr[3];
  TGraphErrors *RlongMtGrErr[3];
  TGraphErrors *RosRatMtGrErr[3];
  TGraphErrors *RosDiffMtGrErr[3];
  TGraphErrors *RlongSqrMtGrErr[3];

  vector<TH1F *> hOutProjVect,hSideProjVect,hLongProjVect;
  vector<TH1F *> hFitOutProjVect,hFitSideProjVect,hFitLongProjVect;

  //For all charges: +, -, sum
  for(int iCharge=0; iCharge<3; iCharge++) {
  for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {

  int iter = iCharge*nKtSumBins + iKtBin;
  HbtFitter *myFitter = DoFit(hNumer.at(iter),hDenom.at(iter),hQinv.at(iter),
  PartType,SourceFunc,NonFemtoFunc);

  TH1F *hExpOutProj = myFitter->Get3DOutProjection();
  TH1F *hExpSideProj = myFitter->Get3DSideProjection();
  TH1F *hExpLongProj = myFitter->Get3DLongProjection();

  TH1F *hFitOutProj = myFitter->Get3DFitOutProjection();
  TH1F *hFitSideProj = myFitter->Get3DFitSideProjection();
  TH1F *hFitLongProj = myFitter->Get3DFitLongProjection();

  hExpOutProj->SetName(Form("hOutProj_%d_%d",iCharge,iKtBin));
  hExpSideProj->SetName(Form("hSideProj_%d_%d",iCharge,iKtBin));
  hExpLongProj->SetName(Form("hLongProj_%d_%d",iCharge,iKtBin));

  hFitOutProj->SetName(Form("hFitOutProj_%d_%d",iCharge,iKtBin));
  hFitSideProj->SetName(Form("hFitSideProj_%d_%d",iCharge,iKtBin));
  hFitLongProj->SetName(Form("hFitLongProj_%d_%d",iCharge,iKtBin));

  hOutProjVect.push_back(hExpOutProj);
  hSideProjVect.push_back(hExpSideProj);
  hLongProjVect.push_back(hExpLongProj);

  hFitOutProjVect.push_back(hFitOutProj);
  hFitSideProjVect.push_back(hFitSideProj);
  hFitLongProjVect.push_back(hFitLongProj);

  //Calculate kT and mT binning
  KtLo=KtShift + FitKtArray[iKtBin][0]*KtStep;
  KtHi=KtShift + FitKtArray[iKtBin][1]*KtStep + KtStep;
  KtVal = (KtHi + KtLo)*0.5;
  MtVal = TMath::Sqrt(KtVal*KtVal + MassSqr);

  KtArr[iKtBin] = KtVal;
  KtErrArr[iKtBin] = 0.01;
  MtArr[iKtBin] = MtVal;

  //Obtain fit parameters
  retParVect.clear();
  retParVect = myFitter->GetFemto3DParameters();

  Chi2Vect.push_back(myFitter->GetChi2PerNDF());
  LamVect.push_back(retParVect.at(0));
  RoutVect.push_back(retParVect.at(1));
  RsideVect.push_back(retParVect.at(2));
  RlongVect.push_back(retParVect.at(3));

#ifdef _DEBUG_
  ///////// DEBUG ///////////
  std::cout << "-------------------------------------------------------------" << std::endl;
  std::cout << "Rout: " << RoutVect.back() << " Rside: " << RsideVect.back() 
      << " Rlong: " << RlongVect.back() << std::endl;
  std::cout << "-------------------------------------------------------------" << std::endl;
#endif

  //Obtain fit error parameters
  retParVect.clear();
  retParVect = myFitter->GetFemto3DParError();

  LamErrVect.push_back(retParVect.at(0));
  RoutErrVect.push_back(retParVect.at(1));
  RsideErrVect.push_back(retParVect.at(2));
  RlongErrVect.push_back(retParVect.at(3));

#ifdef _DEBUG_
  ///////// DEBUG ///////////
  std::cout << "-------------------------------------------------------------" << std::endl;
  std::cout << "RoutErr: " << RoutErrVect.back() << " RsideErr: " << RsideErrVect.back() 
      << " RlongErr: " << RlongErrVect.back() << std::endl;
  std::cout << "-------------------------------------------------------------" << std::endl;
#endif

  //Parameters for the given charge and each kT bin
  Chi2Arr[iCharge][iKtBin] = Chi2Vect.back();
  LamArr[iKtBin] = LamVect.back();
  LamErrArr[iKtBin] = LamErrVect.back();
  RoutArr[iKtBin] = RoutVect.back();
  RoutErrArr[iKtBin] = RoutErrVect.back();
  RsideArr[iKtBin] = RsideVect.back();
  RsideErrArr[iKtBin] = RsideErrVect.back();
  RlongArr[iKtBin] = RlongVect.back();
  RlongErrArr[iKtBin] = RlongErrVect.back();
  RosRatArr[iKtBin] = RoutArr[iKtBin]/RsideArr[iKtBin];

  //         RosRatErrArr[iKtBin] = ( (RoutArr[iKtBin] * RsideErrArr[iKtBin] +
  //         RsideArr[iKtBin] * RoutErrArr[iKtBin]) /
  //         (RsideArr[iKtBin] * RsideArr[iKtBin]) );         //error(x/y) = (x*error(y)+y*error(x))/y^2

  RosRatErrArr[iKtBin] = TMath::Sqrt( (RoutErrArr[iKtBin] * RoutErrArr[iKtBin]) /
                                     (RsideArr[iKtBin] * RsideArr[iKtBin]) +
                                     (RoutArr[iKtBin] * RoutArr[iKtBin] * 
                                      RsideErrArr[iKtBin] * RsideErrArr[iKtBin]) /
                                     (RsideArr[iKtBin] * RsideArr[iKtBin] *
                                      RsideArr[iKtBin] * RsideArr[iKtBin]) );  //error(x/y) = sqrt( (error(x)/y)^2 + (x*error(y)/y^2)^2)
  RosDiffArr[iKtBin] = (RoutArr[iKtBin] * RoutArr[iKtBin] - 
                        RsideArr[iKtBin] * RsideArr[iKtBin]);               
  RosDiffErrArr[iKtBin] = (2. * RoutArr[iKtBin] * RoutErrArr[iKtBin] +
                           2. * RsideArr[iKtBin] * RsideErrArr[iKtBin]);    //error(x^2-y^2) = 2*x*error(x) + 2*y*error(y)
  RlongSqrArr[iKtBin] = RlongArr[iKtBin] * RlongArr[iKtBin];
  RlongSqrErrArr[iKtBin] = (2. * RlongArr[iKtBin] * RlongErrArr[iKtBin]);   //error(x^n) = n * x^(n-1) * error(x)

} //for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++)


#ifdef _DEBUG_
///////// DEBUG ///////////
std::cout << "Rout array" << std::endl;
for(int iKt=0; iKt<nKtSumBins; iKt++) {
    std::cout << RoutArr[iKt] << "\t";
}
std::cout << std::endl;
std::cout << "Rout error array" << std::endl;
for(int iKt=0; iKt<nKtSumBins; iKt++) {
    std::cout << RoutErrArr[iKt] << "\t";
}
std::cout << std::endl;
#endif

//Lambda
LamKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,LamArr,KtErrArr,LamErrArr);
LamKtGrErr[iCharge]->SetName(Form("LamKtGrErr_%d",iCharge));
SetTGraphStyle(LamKtGrErr[iCharge], iCharge);
LamKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
LamKtGrErr[iCharge]->GetYaxis()->SetTitle("#lambda");
LamMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,LamArr,KtErrArr,LamErrArr);
LamMtGrErr[iCharge]->SetName(Form("LamMtGrErr_%d",iCharge));
SetTGraphStyle(LamMtGrErr[iCharge], iCharge);
LamMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
LamMtGrErr[iCharge]->GetYaxis()->SetTitle("#lambda");

//Rout
RoutKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RoutArr,KtErrArr,RoutErrArr);
RoutKtGrErr[iCharge]->SetName(Form("RoutKtGrErr_%d",iCharge));
SetTGraphStyle(RoutKtGrErr[iCharge], iCharge);
RoutKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RoutKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{out} (fm)");
RoutMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RoutArr,KtErrArr,RoutErrArr);
RoutMtGrErr[iCharge]->SetName(Form("RoutMtGrErr_%d",iCharge));
SetTGraphStyle(RoutMtGrErr[iCharge], iCharge);
RoutMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RoutMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{out} (fm)");

//Rside
RsideKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RsideArr,KtErrArr,RsideErrArr);
RsideKtGrErr[iCharge]->SetName(Form("RsideKtGrErr_%d",iCharge));
SetTGraphStyle(RsideKtGrErr[iCharge], iCharge);
RsideKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RsideKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{side} (fm)");
RsideMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RsideArr,KtErrArr,RsideErrArr);
RsideMtGrErr[iCharge]->SetName(Form("RsideMtGrErr_%d",iCharge));
SetTGraphStyle(RsideMtGrErr[iCharge], iCharge);
RsideMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RsideMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{side} (fm)");

//Rlong
RlongKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RlongArr,KtErrArr,RlongErrArr);
RlongKtGrErr[iCharge]->SetName(Form("RlongKtGrErr_%d",iCharge));
SetTGraphStyle(RlongKtGrErr[iCharge], iCharge);
RlongKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RlongKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{long} (fm)");
RlongMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RlongArr,KtErrArr,RlongErrArr);
RlongMtGrErr[iCharge]->SetName(Form("RlongMtGrErr_%d",iCharge));
SetTGraphStyle(RlongMtGrErr[iCharge], iCharge);
RlongMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RlongMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{long} (fm)");

//RosRat
RosRatKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RosRatArr,KtErrArr,RosRatErrArr);
RosRatKtGrErr[iCharge]->SetName(Form("RosRatKtGrErr_%d",iCharge));
SetTGraphStyle(RosRatKtGrErr[iCharge], iCharge);
RosRatKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RosRatKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{o}/R_{s}");
RosRatMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RosRatArr,KtErrArr,RosRatErrArr);
RosRatMtGrErr[iCharge]->SetName(Form("RosRatMtGrErr_%d",iCharge));
SetTGraphStyle(RosRatMtGrErr[iCharge], iCharge);
RosRatMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RosRatMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{o}/R_{s}");

//RosDiff
RosDiffKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RosDiffArr,KtErrArr,RosDiffErrArr);
RosDiffKtGrErr[iCharge]->SetName(Form("RosDiffKtGrErr_%d",iCharge));
SetTGraphStyle(RosDiffKtGrErr[iCharge], iCharge);
RosDiffKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RosDiffKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{o}^{2}-R_{s}^{2} (fm^{2})");
RosDiffMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RosDiffArr,KtErrArr,RosDiffErrArr);
RosDiffMtGrErr[iCharge]->SetName(Form("RosDiffMtGrErr_%d",iCharge));
SetTGraphStyle(RosDiffMtGrErr[iCharge], iCharge);
RosDiffMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RosDiffMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{o}^{2}-R_{s}^{2} (fm^{2})");

//RlongSqr
RlongSqrKtGrErr[iCharge] = new TGraphErrors(nKtSumBins,KtArr,RlongSqrArr,KtErrArr,RlongErrArr);
RlongSqrKtGrErr[iCharge]->SetName(Form("RlongSqrKtGrErr_%d",iCharge));
SetTGraphStyle(RlongSqrKtGrErr[iCharge],iCharge);
RlongSqrKtGrErr[iCharge]->GetXaxis()->SetTitle("k_{T} (GeV/c)");
RlongSqrKtGrErr[iCharge]->GetYaxis()->SetTitle("R_{l}^{2} (fm^{2})");
RlongSqrMtGrErr[iCharge] = new TGraphErrors(nKtSumBins,MtArr,RlongSqrArr,KtErrArr,RlongErrArr);
RlongSqrMtGrErr[iCharge]->SetName(Form("RlongSqrMtGrErr_%d",iCharge));
SetTGraphStyle(RlongSqrMtGrErr[iCharge],iCharge);
RlongSqrMtGrErr[iCharge]->GetXaxis()->SetTitle("m_{T} (GeV/c^{2})");
RlongSqrMtGrErr[iCharge]->GetYaxis()->SetTitle("R_{l}^{2} (fm^{2})");
} //for(unsigned int iCharge=0; iCharge<3; iCharge++)

TFile *oFile = new TFile(oFileName.Data(),"recreate");

for(unsigned int iHisto = 0; iHisto<hOutProjVect.size(); iHisto++) {
    SetProjectionStyle(hOutProjVect.at(iHisto),0);
    hOutProjVect.at(iHisto)->Write();
    SetProjectionStyle(hSideProjVect.at(iHisto),1);
    hSideProjVect.at(iHisto)->Write();
    SetProjectionStyle(hLongProjVect.at(iHisto),2);
    hLongProjVect.at(iHisto)->Write();

    SetProjectionStyle(hFitOutProjVect.at(iHisto),0,true);
    hFitOutProjVect.at(iHisto)->Write();
    SetProjectionStyle(hFitSideProjVect.at(iHisto),1,true);
    hFitSideProjVect.at(iHisto)->Write();
    SetProjectionStyle(hFitLongProjVect.at(iHisto),2,true);
    hFitLongProjVect.at(iHisto)->Write();
}

for(int iCharge=0; iCharge<3; iCharge++) {
    LamKtGrErr[iCharge]->Write();
    LamMtGrErr[iCharge]->Write();
    RoutKtGrErr[iCharge]->Write();
    RoutMtGrErr[iCharge]->Write();
    RsideKtGrErr[iCharge]->Write();
    RsideMtGrErr[iCharge]->Write();
    RlongKtGrErr[iCharge]->Write();
    RlongMtGrErr[iCharge]->Write();
    RosRatKtGrErr[iCharge]->Write();
    RosRatMtGrErr[iCharge]->Write();
    RosDiffKtGrErr[iCharge]->Write();
    RosDiffMtGrErr[iCharge]->Write();
    RlongSqrKtGrErr[iCharge]->Write();
    RlongSqrMtGrErr[iCharge]->Write();
}

ofstream oTxtFile;
oTxtFile.open(oTxtFileName.Data());
oTxtFile << Form("Collision energy: %3i \t Centrality: %1i \t Coulomb R: %2.1f \t Systematics: %d\n",
                 energy, cent, CoulombSR, syst);
oTxtFile << "-------------------------------\n";

oTxtFile << Form("# kT bins: %d \t kT low: %3.2f \t kT hi: %3.2f \n", NKtBins,0.1,1.2);

oTxtFile << "kT bins: ";
for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
    KtLo=KtShift + FitKtArray[iKtBin][0]*KtStep;
    KtHi=KtShift + FitKtArray[iKtBin][1]*KtStep + KtStep;
    oTxtFile << Form("%3.2f-%3.2f",KtLo,KtHi);
    if(iKtBin == (nKtSumBins-1)) {
        oTxtFile << "\n";
    }
    else {
        oTxtFile << " | ";
    }
}
oTxtFile << "mT bins: ";
for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
    KtLo=KtShift + FitKtArray[iKtBin][0]*KtStep;
    KtHi=KtShift + FitKtArray[iKtBin][1]*KtStep + KtStep;
    float MtLo = TMath::Sqrt(KtLo*KtLo + MassSqr);
    float MtHi = TMath::Sqrt(MtLo*MtLo + MassSqr);
    oTxtFile << Form("%3.2f-%3.2f",MtLo,MtHi);
    if(iKtBin == (nKtSumBins-1)) {
        oTxtFile << "\n";
    }
    else {
        oTxtFile << " | ";
    }
}

int point;
Double_t x,y,ey;

//Write to file
for(int iCharge=0; iCharge<3; iCharge++) {

    switch(iCharge) {
        case 0:
            oTxtFile << "Positive charge: \n";
            break;
        case 1:
            oTxtFile << "Negative charge: \n";
            break;
        case 2:
            oTxtFile << "Sum of charges: \n";
            break;
        default:
            oTxtFile << "Error in the charge cycle \n";
            break;
    };

    oTxtFile << "-------------------------------\n";

    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "chi2/ndf: ";
        }
        oTxtFile << Form(" %5.4f", Chi2Arr[iCharge][iKtBin]);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "lambda: ";
        }
        point = LamKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  LamKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %5.4f +- %5.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "Rout  : ";
        }
        point = RoutKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RoutKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %5.4f +- %5.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "Rside : ";
        }
        point = RsideKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RsideKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %5.4f +- %5.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "Rlong : ";
        }
        point = RlongKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RlongKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %5.4f +- %5.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "RosRat : ";
        }
        point = RosRatKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RosRatKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %5.4f +- %5.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "RosDiff : ";
        }
        point = RosDiffKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RosDiffKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %6.4f +- %6.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    for(int iKtBin=0; iKtBin<nKtSumBins; iKtBin++) {
        if(iKtBin==0) {
            oTxtFile << "Rlong^2 : ";
        }
        point = RlongSqrKtGrErr[iCharge]->GetPoint(iKtBin,x,y);
        ey =  RlongSqrKtGrErr[iCharge]->GetErrorY(iKtBin);
        oTxtFile << Form(" %6.4f +- %6.4f ", y, ey);
        if(iKtBin == (nKtSumBins-1)) {
            oTxtFile << "\n";
        }
        else {
            oTxtFile << " | ";
        }
    }
    oTxtFile << "-------------------------------\n";
} //for(int iCharge=0; iCharge<3; iCharge++)

oTxtFile << "\n";
oTxtFile.close();

oFile->Write();
oFile->Close();
*/
}

//--------------------------------------------------------------------------
HbtFitter *DoFitMc(TH1F* hNum, TH1F* hDen,
                   TH1F *hNumMc, TH1F *hDenMc,
                   HbtFitter *fitter, float coulRad)
{
    //
    // General options for the fitter
    //
    fitter->SetFitMethod(1);  // log-likelihood
    fitter->SetSourceForm(0);
    fitter->SetCorrectOnMc(false);
    fitter->SetProjectionType(false); // project by vals
    fitter->Set1DBinsNum(qInvBins);
    fitter->Set1DHistoRange(qInvLo, qInvHi);
    fitter->SetCoulombSourceRadius(coulRad);
    fitter->Set1DNumer(hNumMc);
    fitter->Set1DDenom(hDenMc);
    fitter->Set1DFitRange(FitRangeLo, FitRangeHi); // set fit range
    //
    // Fit MC
    //
    cout << "***** Starting to fit to the MC CF *****";
    fitter->UseCoulombCorrection(false);  // on/off Coulomb correction
    fitter->SetInit1DFitParameters(1.0 /* lambda */,
                                   0.0 /* R */,
                                   0.1 /* N */,
                                   1.0 /* a */,
                                   0.0 /* b */);
    fitter->SetInit1DFitParErrors(1.0e-6, 1.0e-6, 1.0e-6, 1.0e-6, 1.0e-6);
    fitter->SetInit1DFitParLimitLo(0.0, 0.0, 0.0, -5.0, -5.0);
    fitter->SetInit1DFitParLimitHi(0.0, 0.0, 0.8, 5.0, 5.0);
    fitter->FixFitParameter1D(0); // fix lambda
    fitter->FixFitParameter1D(1); // fix R
    fitter->Fit1D(); // fit to the CF

    vector<double> nonFemtoPar = fitter->GetNonFemto1DParameters();
    cout << "Chi2: " << fitter->Get1dChi2() << " NDF: " << fitter->Get1dNDF()
         << " Chi2/NDF: " << fitter->Get1dChi2PerNDF() << endl;
#if 1
    fitter->Clear();
    //
    // Fit to the real CF
    //
    fitter->Set1DFitRange(FitRangeLo, FitRangeHi); // set fit range
    fitter->SetInit1DFitParameters(1.0 /* lambda */,
                                   0.0 /* R */,
                                   0.1 /* N */,
                                   nonFemtoPar[1] /* a */,
                                   nonFemtoPar[2] /* b */);
    fitter->SetInit1DFitParErrors(1.0e-6, 1.0e-6, 1.0e-6, 1.0e-6, 1.0e-6);
    fitter->SetInit1DFitParLimitLo(0.1, 0.3, 0.0, 0.0, 0.0);
    fitter->SetInit1DFitParLimitHi(1.0, 1.5, 1.3, 0.0, 0.0);
    fitter->UseCoulombCorrection(false);  // on/off Coulomb correction
    fitter->FixFitParameter1D(3);
    fitter->FixFitParameter1D(4);
    fitter->Set1DNumer(hNum);
    fitter->Set1DDenom(hDen);

    fitter->Fit1D();
#endif
    fitter->Make1DFunctions();
    cout << "Chi2: " << fitter->Get1dChi2() << " NDF: " << fitter->Get1dNDF()
         << " Chi2/NDF: " << fitter->Get1dChi2PerNDF() << endl;

    return fitter;
}
//--------------------------------------------------------------------------
HbtFitter *DoFit(TH1F* hNum, TH1F* hDen,
                 int aSourceFunc, int aNonFemtoFunc)
{
    HbtFitter *fitter = new HbtFitter(4, aSourceFunc, aNonFemtoFunc, false);
    fitter->SetFitMethod(1);  // log-likelihood
    fitter->SetSourceForm(0);
    fitter->SetCorrectOnMc(false);
    fitter->SetProjectionType(false); // project by vals
    fitter->Set1DBinsNum(qInvBins);
    fitter->Set1DHistoRange(qInvLo, qInvHi);
    fitter->UseCoulombCorrection(false);  // on/off Coulomb correction
    fitter->SetCoulombSourceRadius(CoulombSR);

    fitter->Set1DNumer(hNum);
    fitter->Set1DDenom(hDen);

    fitter->Set1DFitRange(FitRangeLo, FitRangeHi); // set fit range
    fitter->SetInit1DFitParameters(0.5, 0.7, 0.1, 0., 0.);
    fitter->SetInit1DFitParErrors(1e-6, 1e-6, 1e-6, 1e-6, 1e-6);
    fitter->SetInit1DFitParLimitLo(0.1, 0.3, 0.0, 0.0, 0.0);
    fitter->SetInit1DFitParLimitHi(1.0, 1.5, 2.0, 0.0, 0.0);
    //  fitter->FixFitParameter1D(4);

    // main magic
    fitter->Fit1D();
    /*
       std::vector<double> femtoPar = fitter->GetFemto1DParameters();
       std::vector<double> nonFemtoPar = fitter->GetNonFemto1DParameters();
       fitter->SetInit1DFitParameters(femtoPar[0], femtoPar[1],
       nonFemtoPar[0], nonFemtoPar[1], nonFemtoPar[2]);
       fitter->Fit1D();
       */
    fitter->Make1DFunctions();
    cout << "Chi2: " << fitter->Get1dChi2() << " NDF: " << fitter->Get1dNDF()
         << " Chi2/NDF: " << fitter->Get1dChi2PerNDF() << endl;

    return fitter;
}

//--------------------------------------------------------------------------
void SetTGraphStyle(TGraphErrors *gr, int plot)
{
    Int_t mColor;
    Int_t mMarkerStyle;
    Double_t mMarkerSize = 1.5;
    Int_t mLineWidth = 2;
    const char *yAxis, *name;

    switch (plot)
    {
        case 0:               // Lambda
            mColor = 2;       // Red
            mLineWidth = 2;
            mMarkerStyle = 29;
            yAxis = "#lambda";
            name = "lambda";
            break;
        case 1:               // R
            mColor = 4;       // Blue
            mLineWidth = 2;
            mMarkerStyle = 29;
            yAxis = "R [fm]";
            name = "R";
            break;
        default:
            mColor = 1;       //Black
            mLineWidth = 2;
            mMarkerStyle = 29;
            yAxis = "#chi^{2}";
            name = "chi2";
    }
    gr->SetLineColor(mColor);
    gr->SetMarkerColor(mColor);
    gr->SetLineWidth(mLineWidth);
    gr->SetMarkerStyle(mMarkerStyle);
    gr->SetMarkerSize(mMarkerSize);
    gr->SetTitle(Form(";k_{T} [GeV/c];%s", yAxis));
    gr->SetName(name);
}
/*
//_________________
void SetProjectionStyle(TH1* h, int proj, Bool_t isFit) {
    TString Xstr,Ystr;
    if(proj == 0) {
        Xstr = "q_{out} (GeV/c)";
        Ystr =  "C(q_{out})";
    }
    else if(proj == 1) {
        Xstr = "q_{side} (GeV/c)";
        Ystr =  "C(q_{side})";
    }
    else if(proj == 2) {
        Xstr = "q_{long} (GeV/c)";
        Ystr =  "C(q_{long})";
    }
    int color;
    if(isFit == false) {
        color = 1;
    }
    else {
        color = 2;
    }

    h->SetLineWidth(2);
    h->SetLineColor(color);
    h->SetMarkerColor(color);
    h->SetAxisRange(0.9, 1.6,"Y");
    h->SetAxisRange(FitRangeLo,FitRangeHi,"X");
    h->GetXaxis()->SetTitle(Xstr.Data());
    h->GetYaxis()->SetTitle(Ystr.Data());

    h->SetNdivisions(505,"X");
    h->SetNdivisions(505,"Y");
}

*/
