/*
  This class implements the fitting basics for different
  HBT studies. Right now it processes the 1-dimentional
  case only. It should be easy to expand it to 3D case.
  For the example of the implementation see TryNewFitter.C
*/
#ifndef HbtFitter_h
#define HbtFitter_h

#include <TH1.h>
#include <TH2.h>
#include <TH3.h>
#include <TF1.h>
#include <TF2.h>
#include <TF3.h>
#include <TMinuit.h>
#include <vector>

//#define MESSAGING

/*
double KK_LONG[40] ={ 0.273707, 0.697738, 0.823735, 0.881777, 0.914707,
		      0.936038, 0.950835, 0.961148, 0.968602, 0.974788,
		      0.979086, 0.982599, 0.985363, 0.987413, 0.989307,
		      0.990824, 0.991961, 0.992976, 0.993674, 0.994269,
		      0.995043, 0.995395, 0.995902, 0.996183, 0.996628,
		      0.996828, 0.997042, 0.997301, 0.997473, 0.997612,
		      0.997758, 0.99783,  0.998072, 0.998223, 0.998325,
		      0.998349, 0.998441, 0.998541, 0.998638, 0.998712,
		      0.998765, 0.998821, 0.998893, 0.998901, 0.998976,
		      0.999042, 0.999062, 0.999104, 0.999157, 0.999139  };


//4fm
double PI_LONG[40] = {  0.516417, 0.839253, 0.917381, 0.950586, 0.967903,
			0.978306, 0.984714, 0.98884, 0.991354, 0.993379,
			0.994595, 0.995465, 0.996283, 0.996711, 0.997174,
			0.997565, 0.997882, 0.998155, 0.998293, 0.998465,
			0.998667, 0.998695, 0.998887, 0.998964, 0.999085,
			0.999136, 0.999163, 0.999259, 0.999287, 0.999344,
			0.999383, 0.999361, 0.999455, 0.999501, 0.999544,
			0.999536, 0.999556, 0.999572, 0.999619, 0.999636 };
*/

//_________________
class HbtFitter {

 public:

  HbtFitter(Int_t PartType=4, Int_t SourceForm=0,
            Int_t NonFemtoForm=0, Bool_t ExcludePoints=false);
  ~HbtFitter();
  void Clear();
  
  static HbtFitter* objHbtFitter;

  static void SetObject(HbtFitter *obj) {objHbtFitter = obj;} //Do I need void?

  static void FitFunc1DChiSquare(Int_t &npar, Double_t *gin, Double_t &f, 
				 Double_t *par, Int_t iflag);
  static void FitFunc1DLogLike(Int_t &npar, Double_t *gin, Double_t &f, 
			       Double_t *par, Int_t iflag);
  static void FitFunc3DChiSquare(Int_t &npar, Double_t *gin, Double_t &f, 
				 Double_t *par, Int_t iflag);
  static void FitFunc3DLogLike(Int_t &npar, Double_t *gin, Double_t &f, 
			       Double_t *par, Int_t iflag);
  static void FitFunc3DNonFemtoChiSquare(Int_t &npar, Double_t *gin, Double_t &f,
					 Double_t *par, Int_t iflag);
  static void FitFunc3DNonFemtoLogLike(Int_t &npar, Double_t *gin, Double_t &f,
				       Double_t *par, Int_t iflag);

  //////////////// Setters /////////////

  //General settings
  void SetNonFemtoForm(Int_t NonFemtoFrom = 0);
  void SetSourceForm(Int_t SourceForm = 0);  //0-Gaus,1-Expo
  void SetPartType(Int_t PartType) {mPartType = PartType;}
  void SetFitMethod(Int_t method = 1); //0-ChiSquare, 1-LogLikelihod; 0 is default

  //Apply non-femto correction or not; true by default
  void SetCorrectOnMc(Bool_t DoCorrect = false);
  
  //Initial parameters of the fit
  //1D fit
  void SetInit1DFitParameters(double lam=0.5, double Rinv=3., double norm = 0.5,
			      double a=0., double b=0.);
  void SetInit1DFitParameter(int iPar, double val);
  void SetInit1DFitParErrors(double lam=0.01, double Rinv=0.01, double norm=0.01,
			     double a=0.01, double b=0.01);
  void SetInit1DFitParameterError(int iPar, double val);
  void SetInit1DFitParLimitLo(double lam=0., double Rinv = 0.1, double norm = 0.,
			      double a=-5., double b=-2.);
  void SetInit1DFitParLimitLo(int iPar, double val);
  void SetInit1DFitParLimitHi(double lam=1.05, double Rinv = 50., double norm = 10.,
			      double a=+5., double b=+2.);
  void SetInit1DFitParLimitHi(int iPar, double val);  

  //3D fit
  void SetInit3DFitParameters(double lam=0.3, double Rout=4.,  double Rside=4., double Rlong=4.,
			      double Ros=0.,  double Rol=0.,   double Rsl=0.,   double norm=0.03,
			      double ao=0.,   double as=0.,    double al=0.,    
			      double bo=0.,   double bs=0.,    double bl=0.);
  void SetInit3DFitParameter(int iPar, double val);

  void SetInit3DFitParErrors(double lam=0.01, double Rout=0.01, double Rside=0.01, double Rlong=0.01,
			     double Ros=0.01, double Rol=0.01,  double Rsl=0.01,   double norm=0.001,
			     double ao=0.01,  double as=0.01,   double al=0.01,
			     double bo=0.01,  double bs=0.01,   double bl=0.01);
  void SetInit3DFitParameterError(int iPar, double val);

  void SetInit3DFitParLimitLo(double lam=0.,  double Rout=0.,   double Rside=0., double Rlong=0.,
			      double Ros=0.,  double Rol=0.,    double Rsl=0.,   double norm=0.,
			      double ao=-5.,  double as=-5.,    double al=-5.,
			      double bo=-2.,  double bs=-2.,   double bl=-2.);
  void SetInit3DFitParLimitLo(int iPar, double val);

  void SetInit3DFitParLimitHi(double lam=1.05, double Rout=15., double Rside=15., double Rlong=15.,
			      double Ros=15.,  double Rol=15.,  double Rsl=15.,   double norm=1000.,
			      double ao=5.,    double as=5.,    double al=5.,
			      double bo=2.,  double bs=2.,   double bl=2.);
  void SetInit3DFitParLimitHi(int iPar, double val);
  
  //Fix fit parameters
  void FixFitParameter1D(Int_t iPar) {mFixParameters1D.push_back(iPar);}
  void FixFitParameter3D(Int_t iPar) {mFixParameters3D.push_back(iPar);}

  void SetCoulombSourceRadius(double val);

  //Call this functions to fit histograms
  void Fit1D();
  void Fit3D();
  void FitNonFemto3D();
 
  //Project fit
  void MakeProjections();    //This function calls three below
  void MakeExpProjections();
  void MakeFitNumerator();
  void MakeFitProjections();

  void MakeNonFemtoProjections();
  void MakeMcProjections();
  void MakeNonFemtoFitNumerator();
  void MakeNonFemtoFitProjections();

  //One-dimensional fit
  void Make1DFunctions();
  void Make1DCorrFctn();
  void Make1DFitFunction();

  //Histogram projections
  void Project3DCorrFctn(TH3F* hNumer, TH3F* hDenom, 
			 TH1F *&hOutProjNum, TH1F *&hSideProjNum, TH1F *&hLongProjNum,
			 TH1F *&hOutProjDen, TH1F *&hSideProjDen, TH1F *&hLongProjDen,
			 Int_t mOutBinLow, Int_t mOutBinHi,
			 Int_t mSideBinLow, Int_t mSideBinHi,
			 Int_t mLongBinLow, Int_t mLongBinHi);

  void Project3DCorrFctn(TH3F* hNumer, TH3F* hDenom, 
			 TH1F *&hOutProjNum, TH1F *&hSideProjNum, TH1F *&hLongProjNum,
			 TH1F *&hOutProjDen, TH1F *&hSideProjDen, TH1F *&hLongProjDen,
			 Double_t mQoutLow, Double_t mQoutH1i,
			 Double_t mQsideLow, Double_t mQsideHi,
			 Double_t mQlongLow, Double_t mQlongHi);

  void Project3DDistribution(TH3F* hQdist, 
			     TH1F *&hQoutProj, TH1F *&hQsideProj, TH1F *&hQlongProj,
			     Int_t mOutBinLow, Int_t mOutBinHi,
			     Int_t mSideBinLow, Int_t mSideBinHi,
			     Int_t mLongBinLow, Int_t mLongBinHi);

  void Project3DDistribution(TH3F* hQdist, 
			     TH1F *&hQoutProj, TH1F *&hQsideProj, TH1F *&hQlongProj,
			     Double_t mQoutLow, Double_t mQoutHi,
			     Double_t mQsideLow, Double_t mQsideHi,
			     Double_t mQlongLow, Double_t mQlongHi);

  //Set number of bins and ranges
  void Set1DHistoRange(Double_t lo, Double_t hi) {mQinvVal[0]=lo; mQinvVal[1]=hi;}
  void Set1DBinsNum(Int_t nbins) {mQinvNbins = nbins;}
  void Set3DHistoRange(Double_t outLo, Double_t outHi,
		       Double_t sideLo, Double_t sideHi,
		       Double_t longLo, Double_t longHi);
  void Set3DBinsNum(Int_t OutNBins, Int_t SideNBins, Int_t LongNBins);

  //Fit ranges
  void Set1DFitRange(Double_t lo=0., Double_t hi=0.4);
  void Set3DFitRange(Double_t qOutLo=0., Double_t qOutHi=0.4,
		     Double_t qSideLo=0., Double_t qSideHi=0.4,
		     Double_t qLongLo=0., Double_t qLongHi=0.4);

  //Set 3D projection ranges
  void SetProjectionType(Bool_t type) {mProjectionType = type;} //Default is by bins
  void SetProjectionBins(Int_t oLow, Int_t oHi, Int_t sLow, Int_t sHi, Int_t lLow, Int_t lHi);
  void SetOutProjectionBins(Int_t oLow, Int_t oHi) {mOutProjBins[0]=oLow; mOutProjBins[1]=oHi;}
  void SetSideProjectionBins(Int_t sLow, Int_t sHi) {mSideProjBins[0]=sLow; mSideProjBins[1]=sHi;}
  void SetLongProjectionBins(Int_t lLow, Int_t lHi) {mLongProjBins[0]=lLow; mLongProjBins[1]=lHi;}

  void SetProjectionVals(Double_t oLo, Double_t oHi, Double_t sLo, Double_t sHi,
			 Double_t lLo, Double_t lHi);
  void SetOutProjectionVals(Double_t lo, Double_t hi) {mOutProjVals[0]=lo; mOutProjVals[1]=hi;}
  void SetSideProjectionVals(Double_t lo, Double_t hi) {mSideProjVals[0]=lo; mSideProjVals[1]=hi;}
  void SetLongProjectionVals(Double_t lo, Double_t hi) {mLongProjVals[0]=lo; mLongProjVals[1]=hi;}

  //In case of the excluding the region from the fit
  void SetExcludeFromFit(Bool_t ExcludePoints) {mExcludeRange = ExcludePoints;}
  void SetExcludeRange(Double_t xMin, Double_t xMax);
  
  //Set non-femtscopic parameters
  void Set1DNonFemtoParameters(vector<double> par);
  void Set3DNonFemtoParameters(vector<double> parOut,
			       vector<double> parSide,
			       vector<double> parLong);

  //Operations on histograms
  void Set1DHistos(TH1* rat, TH1* num, TH1* den);
  void Set1DNumer(TH1* histo) {h1dNumer = (TH1F*)histo;}
  void Set1DDenom(TH1* histo) {h1dDenom = (TH1F*)histo;}
  void Set1DCorrFctn(TH1* histo) {h1dCorrFctn = (TH1F*)histo;}
  void Set1DMcHistos(TH1* rat, TH1* num, TH1* den);
  void Set1DMcNumer(TH1* histo) {h1dMcNumer = (TH1F*)histo;}
  void Set1DMcDenom(TH1* histo) {h1dMcDenom = (TH1F*)histo;}
  void Set1DMcCorrFctn(TH1* histo) {h1dMcCorrFctn = (TH1F*)histo;}

  void Set3DHistos(TH3* hCorr, TH3* hNum, TH3* hDen, TH3* hCoul);
  void Set3DNumer(TH3* histo) {h3dNumer = (TH3F*)histo;}
  void Set3DDenom(TH3* histo) {h3dDenom = (TH3F*)histo;}
  void Set3DCorrFctn(TH3* histo) {h3dCorrFctn = (TH3F*)histo;}
  void Set3DCoulomb(TH3* histo) {h3dCoulombWeight = (TH3F*) histo;}
  void Set3DMcHistos(TH3* rat, TH3* num, TH3* den);
  void Set3DMcNumer(TH3* histo) {h3dMcNumer = (TH3F*)histo;}
  void Set3DMcDenom(TH3* histo) {h3dMcDenom = (TH3F*)histo;}
  void Set3DMcCorrFctn(TH3* histo) {h3dMcCorrFctn = (TH3F*)histo;}

  /////////////////// Getters /////////////////////

  TH1F* Get1DNumer() {return h1dNumer;}
  TH1F* Get1DDenom() {return h1dDenom;}
  TH1F* Get1DCorrFctn() {return h1dCorrFctn;}
  TH1F* Get1DMcNumer() {return h1dMcNumer;}
  TH1F* Get1DMcDenom() {return h1dMcDenom;}
  TH1F* Get1DMcCorrFctn() {return h1dMcCorrFctn;}
  TH1F* Get1DCorrFctnFit() {return h1dFitNumerator;}

  TH3F* Get3DNumer() {return h3dNumer;}
  TH3F* Get3DDenom() {return h3dDenom;}
  TH3F* Get3DCorrFctn() {return h3dCorrFctn;}
  TH3F* Get3DCoulombWeight() {return h3dCoulombWeight;}
  TH3F* Get3DMcNumer() {return h3dMcNumer;}
  TH3F* Get3DMcDenom() {return h3dMcDenom;}
  TH3F* Get3DMcCorrFctn() {return h3dMcCorrFctn;}

  //Femtoscopic projections
  TH1F* Get3DOutProjection() {return h3dCorrFctnOutProj;}
  TH1F* Get3DFitOutProjection() {return h3dCorrFctnFitOutProj;}
  TH1F* Get3DSideProjection() {return h3dCorrFctnSideProj;}
  TH1F* Get3DFitSideProjection() {return h3dCorrFctnFitSideProj;}
  TH1F* Get3DLongProjection() {return h3dCorrFctnLongProj;}
  TH1F* Get3DFitLongProjection() {return h3dCorrFctnFitLongProj;}

  //Non-femtoscopic projections
  TH1F* Get3DNonFemtoOutProjection() {return h3dMcCorrFctnOutProj;}
  TH1F* Get3DNonFemtoFitOutProjection() {return h3dCorrFctnNonFemtoFitOutProj;}
  TH1F* Get3DNonFemtoSideProjection() {return h3dMcCorrFctnSideProj;}
  TH1F* Get3DNonFemtoFitSideProjection() {return h3dCorrFctnNonFemtoFitSideProj;}
  TH1F* Get3DNonFemtoLongProjection() {return h3dMcCorrFctnLongProj;}
  TH1F* Get3DNonFemtoFitLongProjection() {return h3dCorrFctnNonFemtoFitLongProj;}

  //Return number of femto and non-femto parameters
  Int_t GetFemto1DParNum() {return mFemto1DParNum;}
  Int_t GetNonFemto1DParNum() {return mNonFemto1DParNum;}
  Int_t GetFemto3DParNum() {return mFemto3DParNum;}
  Int_t GetNonFemto3DParNum() {return mNonFemto3DParNum;}

  //Return fit parameters
  double Get1dChi2() {return m1dChi2;}
  double Get1dNDF() {return m1dNDF;}
  double Get1dChi2PerNDF() {return m1dChi2PerNDF;}
  vector<double> GetFemto1DParameters() {return mFemto1DVal;}
  vector<double> GetFemto1DParError() {return mFemto1DValErr;}
  vector<double> GetNonFemto1DParameters() {return mNonFemto1DVal;}
  vector<double> GetNonFemto1DParError() {return mNonFemto1DValErr;}

  double GetChi2() {return mChi2;}
  double GetNDF() {return mNDF;}
  double GetChi2PerNDF() {return mChi2PerNDF;}
  vector<double> GetFemto3DParameters() {return mFemto3DVal;}
  vector<double> GetFemto3DParError() {return mFemto3DValErr;}
  vector<double> GetNonFemto3DParameters() {return mNonFemto3DVal;}
  vector<double> GetNonFemto3DParError() {return mNonFemto3DValErr;}

  //Coulomb corrections
  Double_t Coulomb1D(Int_t qInvBin);
  Double_t Coulomb3D(Int_t qOutBin, Int_t qSideBin, Int_t qLongBin);

  //Theoretical correlation functions
  Double_t TheorCorrFctn1D(Double_t *par, Int_t qInvBin);
  Double_t TheorCorrFctn3D(Double_t *par, Int_t qOutBin, Int_t qSideBin, Int_t qLongBin);

  //Non-femtoscopic functions
  Double_t NonFemto1D(Double_t *par, Double_t qInv);
  Double_t NonFemto3D(Double_t *par, Double_t qOut, Double_t qSide, Double_t qLong);

  //Use Coulomb correction: Correct if true, if not Coulomb=1
  void UseCoulombCorrection(Bool_t IsUsed) {mUseCoulombCorrection = IsUsed;}

  //Static minimization functions for TMinuit
  Double_t CorrFctn1DChiSqr(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
  Double_t CorrFctn1DLogLike(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
  Double_t CorrFctn3DLogLike(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
  Double_t CorrFctn3DChiSqr(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
  Double_t CorrFctn3DNonFemtoLogLike(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);
  Double_t CorrFctn3DNonFemtoChiSqr(Int_t &npar, Double_t *gin, Double_t &f, Double_t *par, Int_t iflag);

 private:

  //General parameters
  Int_t    mPartType;      //3-pion, 4-kaon
  Int_t    mNonFemtoForm;  //0-unity, 1-pol1, 2-pol2, 3-sqrt(pol2), 4-gaussian, 5-a/(1+bQ^2)
  Int_t    mSourceForm;    //0-gaussian, 1-expoential

  //To use or not to use Coulomb correction. To use by default
  Bool_t   mUseCoulombCorrection;
  Int_t    mFitMethod;     //0-ChiSquare, 1-LogLikelihood

  Double_t mCoulombSR;
  Int_t    mCoulombCorrBin;   // !!! Depends on the dimension of the CoulombCorrectionArray

  //Correct on MC or not. Default is true
  Bool_t   mCorrectOnMc;

  //Histogram projection by bins or by vals
  Bool_t   mProjectionType;

  //Set parameters that have to be fixed
  vector<Int_t> mFixParameters1D;
  vector<Int_t> mFixParameters3D;
  
  //If exlude points from fit
  Bool_t   mExcludeRange;

  //Number of bins in histos
  Int_t    mQinvNbins;      //number of bins in 1D histogram
  Int_t    mQoutNbins;      //Out
  Int_t    mQsideNbins;     //Side
  Int_t    mQlongNbins;     //Long

  //Histogram ranges
  Double_t  mQinvVal[2];     //qInv Lo,Hi
  Double_t  mQoutVal[2];     //qOut Lo,Hi
  Double_t  mQsideVal[2];    //qSide Lo,Hi
  Double_t  mQlongVal[2];    //qLong Lo,Hi

  //Fit ranges
  Int_t     mQinvFitRangeBin[2];
  Double_t  mQinvFitRange[2];
  Int_t     mQoutFitRangeBin[2];
  Double_t  mQoutFitRange[2];
  Int_t     mQsideFitRangeBin[2];
  Double_t  mQsideFitRange[2];
  Int_t     mQlongFitRangeBin[2];
  Double_t  mQlongFitRange[2];

  //Projection ranges in bins (from bin[0] to bin[1])
  Int_t     mOutProjBins[2];
  Int_t     mSideProjBins[2];
  Int_t     mLongProjBins[2];
  //Projection ranges in values (from val[0] to val[1])
  Double_t  mOutProjVals[2];
  Double_t  mSideProjVals[2];
  Double_t  mLongProjVals[2];

  //Number of the non-femtosocpic parameters
  Int_t     mFemto1DParNum;
  Int_t     mFemto3DParNum;
  Int_t     mNonFemto1DParNum;
  Int_t     mNonFemto3DParNum;

  //Fit parameters (after fit)
  double    m1dChi2;
  double    m1dNDF;
  double    m1dChi2PerNDF;
  vector<double> mFemto1DVal;
  vector<double> mFemto1DValErr;
  vector<double> mNonFemto1DVal;
  vector<double> mNonFemto1DValErr;
  double mInit1DFitVal[5];
  double mInit1DFitValErr[5];
  double mInit1DFitValLo[5];
  double mInit1DFitValHi[5];

  double    mChi2;
  double    mNDF;
  double    mChi2PerNDF;
  vector<double> mFemto3DVal;
  vector<double> mFemto3DValErr;
  vector<double> mNonFemto3DVal;
  vector<double> mNonFemto3DValErr;
  double mInit3DFitVal[14];
  double mInit3DFitValErr[14];
  double mInit3DFitValLo[14];
  double mInit3DFitValHi[14];

  //Values of the 1D baseline non-femtoscopic parameters
  Double_t  mNonFemto1DParArr[3];
  Double_t  mNonFemto1DParErrArr[3];
  //Values of the 3D baseline non-femtoscopic parameters [out,side,long][mNonFemtoParNum]
  Double_t  mNonFemto3DParArr[3][3];
  Double_t  mNonFemto3DParErrArr[3][3];  
  
  //Region that is excluded from the fit: 1D case
  Double_t  mExcludedVal[2];
  //Region that is excluded from the fit: 3D case
  Double_t  mExcluded3DVal[3][2];

  //Histograms with Qinv weights. Use Qinv in order to get 1D Coulomb correction
  TH3F*    hBP3DCoulWeightQinv;

  //////////////////
  //  Histograms  //
  //////////////////

  //One-dimensional histograms
  TH1F*    h1dNumer;
  TH1F*    h1dDenom;
  TH1F*    h1dCorrFctn;
  TH1F*    h1dMcNumer;
  TH1F*    h1dMcDenom;
  TH1F*    h1dMcCorrFctn;

  TH1F*    h1dFitNumerator;

  //Three-dimensional histograms
  TH3F*    h3dNumer;
  TH3F*    h3dDenom;
  TH3F*    h3dCorrFctn;
  TH3F*    h3dCoulombWeight;
  TH3F*    h3dMcNumer;
  TH3F*    h3dMcDenom;
  TH3F*    h3dMcCorrFctn;

  //Femtoscopic fit numerator and fit projections
  TH3F*    h3dFitNumerator;
  TH1F*    h3dFitQoutProj;
  TH1F*    h3dFitQsideProj;
  TH1F*    h3dFitQlongProj;
  //Non-femtoscopic fit numerator and fit projections
  TH3F*    h3dNonFemtoFitNumerator;
  TH1F*    h3dNonFemtoFitQoutProj;
  TH1F*    h3dNonFemtoFitQsideProj;
  TH1F*    h3dNonFemtoFitQlongProj;

  //Projected numerators and denominators
  TH1F*    h3dQoutProjNum;
  TH1F*    h3dQoutProjDen;
  TH1F*    h3dMcQoutProjNum;
  TH1F*    h3dMcQoutProjDen;

  TH1F*    h3dQsideProjNum;
  TH1F*    h3dQsideProjDen;
  TH1F*    h3dMcQsideProjNum;
  TH1F*    h3dMcQsideProjDen;
  
  TH1F*    h3dQlongProjNum;
  TH1F*    h3dQlongProjDen;
  TH1F*    h3dMcQlongProjNum;
  TH1F*    h3dMcQlongProjDen;

  //Correlation function projections
  TH1F*    h3dCorrFctnOutProj;
  TH1F*    h3dMcCorrFctnOutProj;
  TH1F*    h3dCorrFctnFitOutProj;
  TH1F*    h3dCorrFctnNonFemtoFitOutProj;

  TH1F*    h3dCorrFctnSideProj;
  TH1F*    h3dMcCorrFctnSideProj;
  TH1F*    h3dCorrFctnFitSideProj;
  TH1F*    h3dCorrFctnNonFemtoFitSideProj;

  TH1F*    h3dCorrFctnLongProj;
  TH1F*    h3dMcCorrFctnLongProj;
  TH1F*    h3dCorrFctnFitLongProj;
  TH1F*    h3dCorrFctnNonFemtoFitLongProj;

  void     Calculate1DChi2(TH1* hCorr, TH1* hFit);
  void     Calculate3DChi2(TH3* hCorr, TH3* hFit);
};

inline void HbtFitter::Set1DHistos(TH1* rat, TH1* num, TH1* den) {
  h1dNumer = (TH1F*)num; h1dDenom = (TH1F*)den; h1dCorrFctn = (TH1F*)rat;
}
inline void HbtFitter::Set1DMcHistos(TH1* rat, TH1* num, TH1* den) {
  h1dMcNumer = (TH1F*)num; h1dMcDenom = (TH1F*)den; h1dMcCorrFctn = (TH1F*)rat;
}
inline void HbtFitter::Set3DHistos(TH3* hCorr, TH3* hNum, TH3* hDen, TH3* hCoul) {
  h3dNumer = (TH3F*)hNum; h3dDenom = (TH3F*)hDen; 
  h3dCorrFctn = (TH3F*)hCorr; h3dCoulombWeight = (TH3F*)hCoul;
}
inline void HbtFitter::Set3DMcHistos(TH3* rat, TH3* num, TH3* den) {
  h3dMcNumer = (TH3F*)num; h3dMcDenom = (TH3F*)den; h3dMcCorrFctn = (TH3F*)rat;
}
inline void HbtFitter::SetProjectionBins(Int_t oLow, Int_t oHi, 
					 Int_t sLow, Int_t sHi, 
					 Int_t lLow, Int_t lHi) {
  mOutProjBins[0]=oLow; mOutProjBins[1]=oHi;
  mSideProjBins[0]=sLow; mSideProjBins[1]=sHi;
  mLongProjBins[0]=lLow; mLongProjBins[1]=lHi;
}
inline void HbtFitter::SetProjectionVals(Double_t oLo, Double_t oHi, 
					 Double_t sLo, Double_t sHi,
					 Double_t lLo, Double_t lHi) {
  mOutProjVals[0] = oLo; mOutProjVals[1] = oHi;
  mSideProjVals[0] = sLo; mSideProjVals[1] = sHi;
  mLongProjVals[0] = lLo; mLongProjVals[1] = lHi;
}

//Coulomb corrections from 0.5 to 6.5 fm with 0.1 fm step
//__________________
double PI_LONG[60][100] = {
  {0.496, 0.807, 0.883, 0.917, 0.936, 0.949, 0.958, 0.964, 0.969, 0.973, 
   0.976, 0.979, 0.981, 0.983, 0.984, 0.986, 0.987, 0.988, 0.989, 0.990, 
   0.990, 0.991, 0.992, 0.992, 0.993, 0.993, 0.994, 0.994, 0.994, 0.995, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.497, 0.808, 0.884, 0.918, 0.938, 0.950, 0.959, 0.965, 0.970, 0.974, 
   0.977, 0.980, 0.982, 0.984, 0.985, 0.987, 0.988, 0.989, 0.990, 0.991, 
   0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 
   0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.497, 0.809, 0.885, 0.919, 0.939, 0.951, 0.960, 0.966, 0.971, 0.975, 
   0.978, 0.981, 0.983, 0.985, 0.986, 0.987, 0.989, 0.990, 0.991, 0.991, 
   0.992, 0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.498, 0.810, 0.886, 0.920, 0.940, 0.952, 0.961, 0.967, 0.972, 0.976, 
   0.979, 0.982, 0.984, 0.985, 0.987, 0.988, 0.990, 0.991, 0.991, 0.992, 
   0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.498, 0.811, 0.887, 0.921, 0.941, 0.953, 0.962, 0.968, 0.973, 0.977, 
   0.980, 0.983, 0.985, 0.986, 0.988, 0.989, 0.990, 0.991, 0.992, 0.993, 
   0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.499, 0.812, 0.888, 0.922, 0.942, 0.954, 0.963, 0.969, 0.974, 0.978, 
   0.981, 0.983, 0.986, 0.987, 0.989, 0.990, 0.991, 0.992, 0.993, 0.993, 
   0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.500, 0.813, 0.889, 0.924, 0.943, 0.955, 0.964, 0.970, 0.975, 0.979, 
   0.982, 0.984, 0.986, 0.988, 0.989, 0.991, 0.992, 0.993, 0.993, 0.994, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.500, 0.813, 0.890, 0.925, 0.944, 0.957, 0.965, 0.971, 0.976, 0.980, 
   0.983, 0.985, 0.987, 0.989, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.501, 0.815, 0.891, 0.926, 0.945, 0.957, 0.966, 0.972, 0.977, 0.981, 
   0.984, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.994, 0.995, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.501, 0.815, 0.892, 0.927, 0.946, 0.958, 0.967, 0.973, 0.978, 0.981, 
   0.984, 0.987, 0.989, 0.990, 0.991, 0.993, 0.993, 0.994, 0.995, 0.995, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.502, 0.816, 0.893, 0.928, 0.947, 0.960, 0.968, 0.974, 0.979, 0.982, 
   0.985, 0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.503, 0.817, 0.894, 0.929, 0.948, 0.961, 0.969, 0.975, 0.980, 0.983, 
   0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.503, 0.818, 0.895, 0.930, 0.949, 0.961, 0.970, 0.976, 0.981, 0.984, 
   0.987, 0.989, 0.990, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.504, 0.819, 0.896, 0.931, 0.950, 0.962, 0.971, 0.977, 0.981, 0.985, 
   0.987, 0.989, 0.991, 0.992, 0.994, 0.994, 0.995, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.504, 0.820, 0.897, 0.932, 0.951, 0.963, 0.972, 0.978, 0.982, 0.985, 
   0.988, 0.990, 0.991, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.505, 0.821, 0.898, 0.933, 0.952, 0.965, 0.972, 0.978, 0.983, 0.986, 
   0.989, 0.990, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.505, 0.822, 0.899, 0.934, 0.953, 0.965, 0.973, 0.979, 0.983, 0.986, 
   0.989, 0.991, 0.992, 0.994, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.506, 0.823, 0.900, 0.935, 0.954, 0.966, 0.974, 0.980, 0.984, 0.987, 
   0.990, 0.991, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 0.997, 0.997, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.507, 0.824, 0.901, 0.936, 0.955, 0.967, 0.975, 0.981, 0.985, 0.988, 
   0.990, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.507, 0.825, 0.902, 0.937, 0.956, 0.968, 0.976, 0.981, 0.985, 0.988, 
   0.991, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.508, 0.826, 0.903, 0.938, 0.957, 0.969, 0.976, 0.982, 0.986, 0.989, 
   0.991, 0.993, 0.994, 0.995, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.508, 0.827, 0.904, 0.939, 0.958, 0.969, 0.977, 0.983, 0.986, 0.989, 
   0.991, 0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.509, 0.828, 0.905, 0.940, 0.959, 0.970, 0.978, 0.983, 0.987, 0.990, 
   0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.509, 0.829, 0.906, 0.941, 0.959, 0.971, 0.979, 0.984, 0.988, 0.990, 
   0.992, 0.993, 0.994, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.510, 0.830, 0.907, 0.942, 0.960, 0.972, 0.979, 0.984, 0.988, 0.991, 
   0.992, 0.994, 0.995, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.511, 0.831, 0.908, 0.942, 0.961, 0.973, 0.980, 0.985, 0.988, 0.991, 
   0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.511, 0.832, 0.909, 0.943, 0.962, 0.973, 0.980, 0.985, 0.989, 0.991, 
   0.993, 0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.512, 0.833, 0.910, 0.944, 0.963, 0.974, 0.981, 0.986, 0.989, 0.991, 
   0.993, 0.994, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.513, 0.833, 0.911, 0.945, 0.963, 0.974, 0.982, 0.986, 0.989, 0.992, 
   0.993, 0.995, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.513, 0.834, 0.912, 0.946, 0.964, 0.975, 0.982, 0.987, 0.990, 0.992, 
   0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.514, 0.835, 0.913, 0.947, 0.965, 0.976, 0.983, 0.987, 0.990, 0.992, 
   0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.514, 0.836, 0.914, 0.948, 0.966, 0.976, 0.983, 0.988, 0.991, 0.993, 
   0.994, 0.995, 0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.515, 0.837, 0.915, 0.948, 0.967, 0.977, 0.984, 0.988, 0.991, 0.993, 
   0.994, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.516, 0.838, 0.916, 0.949, 0.967, 0.978, 0.984, 0.988, 0.991, 0.993, 
   0.994, 0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.516, 0.839, 0.917, 0.950, 0.968, 0.978, 0.985, 0.989, 0.992, 0.993, 
   0.994, 0.995, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.517, 0.840, 0.918, 0.951, 0.969, 0.979, 0.985, 0.989, 0.991, 0.993, 
   0.995, 0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.518, 0.841, 0.918, 0.952, 0.969, 0.979, 0.986, 0.989, 0.992, 0.994, 
   0.995, 0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.518, 0.842, 0.920, 0.952, 0.970, 0.980, 0.986, 0.990, 0.992, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.518, 0.842, 0.920, 0.953, 0.971, 0.980, 0.986, 0.990, 0.992, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.519, 0.844, 0.921, 0.954, 0.971, 0.981, 0.986, 0.990, 0.992, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.520, 0.845, 0.922, 0.955, 0.972, 0.981, 0.987, 0.990, 0.993, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.520, 0.846, 0.923, 0.956, 0.972, 0.982, 0.987, 0.991, 0.993, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.521, 0.847, 0.924, 0.956, 0.973, 0.982, 0.988, 0.991, 0.993, 0.994, 
   0.995, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.521, 0.847, 0.925, 0.957, 0.973, 0.982, 0.988, 0.991, 0.993, 0.995, 
   0.996, 0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.522, 0.848, 0.926, 0.958, 0.974, 0.983, 0.988, 0.991, 0.993, 0.995, 
   0.996, 0.996, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.523, 0.849, 0.926, 0.959, 0.974, 0.983, 0.988, 0.991, 0.993, 0.995, 
   0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.523, 0.850, 0.927, 0.959, 0.975, 0.984, 0.989, 0.991, 0.994, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.524, 0.851, 0.928, 0.960, 0.975, 0.984, 0.989, 0.992, 0.993, 0.995, 
   0.996, 0.996, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.524, 0.851, 0.929, 0.961, 0.976, 0.984, 0.989, 0.992, 0.994, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.525, 0.852, 0.930, 0.961, 0.976, 0.985, 0.989, 0.992, 0.994, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.526, 0.853, 0.930, 0.962, 0.977, 0.985, 0.990, 0.992, 0.994, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.527, 0.854, 0.931, 0.962, 0.978, 0.985, 0.990, 0.993, 0.994, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.527, 0.856, 0.932, 0.963, 0.978, 0.985, 0.990, 0.993, 0.995, 0.995, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.528, 0.856, 0.933, 0.964, 0.978, 0.986, 0.990, 0.993, 0.994, 0.996, 
   0.996, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.528, 0.857, 0.933, 0.964, 0.978, 0.986, 0.990, 0.993, 0.995, 0.996, 
   0.996, 0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.529, 0.858, 0.934, 0.965, 0.979, 0.987, 0.991, 0.993, 0.995, 0.996, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.530, 0.859, 0.935, 0.965, 0.979, 0.986, 0.991, 0.993, 0.995, 0.996, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.530, 0.860, 0.936, 0.966, 0.980, 0.987, 0.991, 0.993, 0.995, 0.996, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.531, 0.861, 0.936, 0.966, 0.980, 0.987, 0.991, 0.993, 0.995, 0.996, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.532, 0.862, 0.937, 0.967, 0.981, 0.987, 0.991, 0.994, 0.995, 0.996, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000}
};

//Kaon Coulomb source sizes
double KK_LONG[60][100] = {
  {0.051, 0.438, 0.629, 0.728, 0.787, 0.827, 0.855, 0.876, 0.892, 0.906, 
   0.916, 0.925, 0.933, 0.939, 0.945, 0.949, 0.953, 0.957, 0.960, 0.963, 
   0.966, 0.968, 0.971, 0.972, 0.974, 0.976, 0.977, 0.979, 0.980, 0.981, 
   0.982, 0.983, 0.984, 0.985, 0.986, 0.987, 0.988, 0.988, 0.989, 0.989, 
   0.990, 0.990, 0.991, 0.991, 0.992, 0.992, 0.992, 0.993, 0.993, 0.993, 
   0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998},

  {0.051, 0.440, 0.631, 0.730, 0.790, 0.830, 0.858, 0.879, 0.896, 0.909, 
   0.920, 0.929, 0.936, 0.942, 0.948, 0.953, 0.957, 0.961, 0.964, 0.966, 
   0.969, 0.971, 0.974, 0.975, 0.977, 0.979, 0.980, 0.982, 0.983, 0.984, 
   0.985, 0.986, 0.987, 0.987, 0.988, 0.989, 0.989, 0.990, 0.990, 0.991, 
   0.992, 0.992, 0.992, 0.993, 0.993, 0.993, 0.994, 0.994, 0.994, 0.994, 
   0.995, 0.995, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.051, 0.442, 0.634, 0.733, 0.794, 0.834, 0.862, 0.883, 0.899, 0.913, 
   0.923, 0.932, 0.939, 0.946, 0.951, 0.956, 0.960, 0.964, 0.967, 0.970, 
   0.972, 0.974, 0.976, 0.978, 0.980, 0.981, 0.983, 0.984, 0.985, 0.986, 
   0.987, 0.988, 0.988, 0.989, 0.990, 0.990, 0.991, 0.991, 0.992, 0.992, 
   0.993, 0.993, 0.994, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.051, 0.444, 0.637, 0.737, 0.797, 0.837, 0.865, 0.887, 0.903, 0.916, 
   0.927, 0.935, 0.943, 0.949, 0.954, 0.959, 0.963, 0.966, 0.970, 0.972, 
   0.975, 0.977, 0.979, 0.980, 0.982, 0.984, 0.984, 0.986, 0.987, 0.988, 
   0.989, 0.989, 0.990, 0.991, 0.991, 0.992, 0.992, 0.993, 0.993, 0.993, 
   0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.051, 0.445, 0.639, 0.739, 0.800, 0.840, 0.869, 0.890, 0.907, 0.920, 
   0.930, 0.939, 0.946, 0.952, 0.957, 0.962, 0.966, 0.969, 0.972, 0.975, 
   0.977, 0.979, 0.981, 0.983, 0.984, 0.985, 0.986, 0.987, 0.988, 0.989, 
   0.990, 0.991, 0.991, 0.992, 0.992, 0.993, 0.993, 0.994, 0.994, 0.994, 
   0.995, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.052, 0.447, 0.642, 0.743, 0.803, 0.844, 0.872, 0.893, 0.910, 0.923, 
   0.933, 0.942, 0.949, 0.955, 0.960, 0.964, 0.968, 0.972, 0.974, 0.977, 
   0.979, 0.981, 0.983, 0.984, 0.986, 0.987, 0.988, 0.989, 0.990, 0.990, 
   0.991, 0.992, 0.992, 0.992, 0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.052, 0.449, 0.644, 0.746, 0.806, 0.847, 0.876, 0.897, 0.913, 0.926, 
   0.936, 0.945, 0.952, 0.958, 0.963, 0.967, 0.971, 0.974, 0.976, 0.979, 
   0.981, 0.983, 0.984, 0.986, 0.987, 0.988, 0.989, 0.989, 0.991, 0.991, 
   0.992, 0.992, 0.993, 0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 
   0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.998, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.052, 0.451, 0.647, 0.749, 0.810, 0.851, 0.879, 0.900, 0.916, 0.929, 
   0.940, 0.948, 0.955, 0.960, 0.965, 0.969, 0.973, 0.976, 0.979, 0.981, 
   0.983, 0.984, 0.986, 0.987, 0.988, 0.989, 0.990, 0.991, 0.991, 0.992, 
   0.993, 0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.052, 0.453, 0.650, 0.752, 0.813, 0.853, 0.882, 0.904, 0.919, 0.932, 
   0.942, 0.951, 0.957, 0.963, 0.968, 0.971, 0.975, 0.978, 0.980, 0.982, 
   0.984, 0.985, 0.987, 0.988, 0.989, 0.990, 0.991, 0.992, 0.992, 0.993, 
   0.993, 0.994, 0.994, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.053, 0.455, 0.653, 0.754, 0.816, 0.857, 0.886, 0.906, 0.922, 0.935, 
   0.945, 0.953, 0.960, 0.965, 0.970, 0.974, 0.976, 0.979, 0.982, 0.984, 
   0.985, 0.987, 0.988, 0.989, 0.990, 0.991, 0.991, 0.992, 0.993, 0.993, 
   0.994, 0.994, 0.995, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.053, 0.457, 0.655, 0.758, 0.819, 0.860, 0.888, 0.909, 0.926, 0.938, 
   0.948, 0.956, 0.962, 0.967, 0.972, 0.975, 0.978, 0.981, 0.983, 0.985, 
   0.986, 0.988, 0.989, 0.990, 0.991, 0.992, 0.992, 0.993, 0.993, 0.994, 
   0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.053, 0.459, 0.657, 0.761, 0.822, 0.863, 0.891, 0.913, 0.928, 0.941, 
   0.950, 0.958, 0.964, 0.969, 0.973, 0.977, 0.980, 0.982, 0.984, 0.986, 
   0.987, 0.989, 0.990, 0.991, 0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 
   0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999},

  {0.053, 0.460, 0.661, 0.764, 0.825, 0.866, 0.894, 0.915, 0.931, 0.943, 
   0.953, 0.960, 0.966, 0.971, 0.975, 0.978, 0.981, 0.984, 0.985, 0.987, 
   0.988, 0.989, 0.990, 0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.994, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 1.000, 1.000},

  {0.053, 0.463, 0.663, 0.767, 0.829, 0.869, 0.897, 0.919, 0.933, 0.946, 
   0.955, 0.962, 0.968, 0.973, 0.977, 0.979, 0.982, 0.985, 0.986, 0.988, 
   0.989, 0.990, 0.991, 0.992, 0.992, 0.993, 0.993, 0.994, 0.995, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.054, 0.464, 0.666, 0.770, 0.832, 0.872, 0.900, 0.921, 0.937, 0.948, 
   0.957, 0.964, 0.970, 0.974, 0.978, 0.981, 0.983, 0.985, 0.987, 0.988, 
   0.990, 0.991, 0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.054, 0.466, 0.668, 0.772, 0.835, 0.876, 0.903, 0.924, 0.939, 0.950, 
   0.959, 0.966, 0.972, 0.975, 0.979, 0.982, 0.984, 0.986, 0.988, 0.989, 
   0.990, 0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.995, 
   0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.054, 0.468, 0.671, 0.776, 0.838, 0.879, 0.906, 0.927, 0.941, 0.952, 
   0.961, 0.967, 0.972, 0.977, 0.980, 0.982, 0.985, 0.987, 0.988, 0.990, 
   0.991, 0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.054, 0.470, 0.674, 0.779, 0.841, 0.881, 0.909, 0.929, 0.943, 0.955, 
   0.963, 0.969, 0.975, 0.978, 0.981, 0.984, 0.986, 0.987, 0.989, 0.990, 
   0.991, 0.992, 0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.055, 0.472, 0.677, 0.781, 0.844, 0.884, 0.911, 0.931, 0.946, 0.957, 
   0.964, 0.971, 0.975, 0.979, 0.982, 0.985, 0.987, 0.988, 0.989, 0.991, 
   0.991, 0.992, 0.993, 0.994, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 0.999, 1.000, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.055, 0.474, 0.679, 0.784, 0.847, 0.887, 0.914, 0.933, 0.947, 0.958, 
   0.966, 0.972, 0.977, 0.980, 0.983, 0.985, 0.987, 0.989, 0.990, 0.991, 
   0.992, 0.993, 0.993, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.055, 0.476, 0.682, 0.787, 0.850, 0.890, 0.916, 0.936, 0.949, 0.960, 
   0.967, 0.973, 0.978, 0.981, 0.984, 0.986, 0.988, 0.989, 0.991, 0.991, 
   0.992, 0.993, 0.994, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.055, 0.478, 0.684, 0.791, 0.853, 0.892, 0.919, 0.938, 0.952, 0.961, 
   0.969, 0.974, 0.979, 0.982, 0.985, 0.986, 0.988, 0.990, 0.991, 0.992, 
   0.993, 0.993, 0.994, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.056, 0.480, 0.687, 0.793, 0.855, 0.895, 0.921, 0.940, 0.953, 0.963, 
   0.970, 0.976, 0.980, 0.983, 0.985, 0.987, 0.989, 0.990, 0.991, 0.992, 
   0.993, 0.993, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.056, 0.482, 0.690, 0.796, 0.858, 0.898, 0.924, 0.941, 0.955, 0.964, 
   0.971, 0.976, 0.980, 0.983, 0.986, 0.988, 0.989, 0.990, 0.992, 0.992, 
   0.993, 0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.056, 0.484, 0.692, 0.799, 0.860, 0.900, 0.926, 0.944, 0.957, 0.966, 
   0.973, 0.977, 0.981, 0.984, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 
   0.993, 0.994, 0.995, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.056, 0.486, 0.695, 0.801, 0.864, 0.903, 0.929, 0.946, 0.958, 0.967, 
   0.974, 0.979, 0.982, 0.984, 0.987, 0.989, 0.990, 0.991, 0.992, 0.993, 
   0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.056, 0.488, 0.698, 0.804, 0.865, 0.906, 0.930, 0.947, 0.960, 0.969, 
   0.975, 0.979, 0.982, 0.985, 0.987, 0.989, 0.990, 0.991, 0.992, 0.993, 
   0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.057, 0.490, 0.701, 0.808, 0.868, 0.907, 0.932, 0.949, 0.961, 0.969, 
   0.975, 0.980, 0.983, 0.986, 0.988, 0.989, 0.990, 0.992, 0.993, 0.994, 
   0.994, 0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.057, 0.492, 0.703, 0.810, 0.871, 0.909, 0.934, 0.951, 0.962, 0.970, 
   0.977, 0.981, 0.984, 0.986, 0.988, 0.989, 0.991, 0.992, 0.993, 0.993, 
   0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.057, 0.494, 0.705, 0.812, 0.873, 0.912, 0.935, 0.953, 0.963, 0.971, 
   0.977, 0.981, 0.984, 0.987, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 
   0.994, 0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 
   0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.057, 0.496, 0.709, 0.816, 0.877, 0.913, 0.938, 0.953, 0.965, 0.973, 
   0.978, 0.982, 0.985, 0.987, 0.989, 0.990, 0.991, 0.992, 0.993, 0.994, 
   0.994, 0.995, 0.996, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.058, 0.497, 0.711, 0.818, 0.878, 0.916, 0.939, 0.955, 0.966, 0.973, 
   0.978, 0.982, 0.985, 0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.994, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.058, 0.500, 0.714, 0.820, 0.882, 0.917, 0.941, 0.956, 0.967, 0.974, 
   0.979, 0.983, 0.986, 0.988, 0.989, 0.991, 0.992, 0.993, 0.993, 0.994, 
   0.995, 0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.058, 0.501, 0.716, 0.823, 0.884, 0.920, 0.943, 0.958, 0.968, 0.975, 
   0.980, 0.984, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 
   0.995, 0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.058, 0.503, 0.718, 0.826, 0.886, 0.922, 0.945, 0.960, 0.970, 0.975, 
   0.980, 0.984, 0.987, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.059, 0.506, 0.721, 0.828, 0.888, 0.924, 0.946, 0.960, 0.969, 0.976, 
   0.981, 0.984, 0.987, 0.989, 0.990, 0.992, 0.993, 0.994, 0.994, 0.995, 
   0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.059, 0.508, 0.723, 0.831, 0.890, 0.925, 0.948, 0.961, 0.970, 0.977, 
   0.981, 0.985, 0.987, 0.989, 0.990, 0.992, 0.993, 0.994, 0.994, 0.995, 
   0.995, 0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.059, 0.510, 0.727, 0.832, 0.892, 0.928, 0.949, 0.962, 0.971, 0.977, 
   0.982, 0.985, 0.988, 0.989, 0.991, 0.992, 0.993, 0.994, 0.994, 0.995, 
   0.995, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.059, 0.511, 0.729, 0.835, 0.895, 0.929, 0.950, 0.963, 0.972, 0.979, 
   0.982, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.994, 0.995, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.060, 0.514, 0.731, 0.838, 0.897, 0.931, 0.951, 0.964, 0.973, 0.979, 
   0.982, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.060, 0.516, 0.734, 0.841, 0.899, 0.933, 0.952, 0.965, 0.974, 0.979, 
   0.983, 0.986, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 
   0.996, 0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.060, 0.518, 0.737, 0.843, 0.900, 0.934, 0.953, 0.967, 0.974, 0.980, 
   0.984, 0.986, 0.988, 0.990, 0.991, 0.993, 0.994, 0.994, 0.995, 0.995, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.060, 0.521, 0.739, 0.845, 0.902, 0.935, 0.955, 0.967, 0.975, 0.980, 
   0.984, 0.986, 0.989, 0.991, 0.992, 0.993, 0.994, 0.994, 0.995, 0.995, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.061, 0.522, 0.742, 0.848, 0.905, 0.936, 0.956, 0.968, 0.976, 0.981, 
   0.984, 0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.061, 0.523, 0.744, 0.850, 0.906, 0.938, 0.957, 0.969, 0.976, 0.981, 
   0.985, 0.987, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.061, 0.526, 0.746, 0.852, 0.908, 0.939, 0.958, 0.969, 0.976, 0.982, 
   0.985, 0.988, 0.990, 0.991, 0.993, 0.993, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.061, 0.528, 0.749, 0.855, 0.910, 0.941, 0.959, 0.969, 0.977, 0.981, 
   0.985, 0.988, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 
   0.996, 0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.062, 0.530, 0.751, 0.856, 0.912, 0.942, 0.960, 0.970, 0.977, 0.982, 
   0.986, 0.987, 0.990, 0.992, 0.993, 0.994, 0.994, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.062, 0.531, 0.754, 0.859, 0.914, 0.944, 0.960, 0.971, 0.978, 0.983, 
   0.986, 0.988, 0.990, 0.992, 0.993, 0.994, 0.994, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.062, 0.533, 0.756, 0.861, 0.914, 0.944, 0.962, 0.971, 0.979, 0.983, 
   0.986, 0.988, 0.990, 0.992, 0.993, 0.994, 0.994, 0.995, 0.996, 0.996, 
   0.996, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.062, 0.536, 0.759, 0.863, 0.918, 0.946, 0.963, 0.973, 0.979, 0.983, 
   0.986, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.063, 0.538, 0.761, 0.865, 0.919, 0.945, 0.963, 0.973, 0.979, 0.984, 
   0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.063, 0.541, 0.764, 0.866, 0.920, 0.947, 0.964, 0.973, 0.980, 0.984, 
   0.987, 0.990, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.998, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.063, 0.543, 0.766, 0.870, 0.921, 0.949, 0.964, 0.974, 0.980, 0.984, 
   0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.063, 0.545, 0.768, 0.871, 0.922, 0.950, 0.965, 0.974, 0.980, 0.984, 
   0.987, 0.989, 0.991, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.996, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.064, 0.547, 0.770, 0.874, 0.924, 0.951, 0.966, 0.975, 0.981, 0.985, 
   0.988, 0.990, 0.991, 0.992, 0.994, 0.994, 0.995, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.064, 0.548, 0.773, 0.876, 0.925, 0.951, 0.967, 0.975, 0.981, 0.985, 
   0.988, 0.990, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.064, 0.551, 0.776, 0.876, 0.926, 0.953, 0.967, 0.976, 0.981, 0.985, 
   0.988, 0.990, 0.992, 0.993, 0.994, 0.995, 0.995, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.064, 0.553, 0.777, 0.878, 0.928, 0.953, 0.968, 0.976, 0.982, 0.986, 
   0.988, 0.991, 0.992, 0.993, 0.994, 0.994, 0.996, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

  {0.065, 0.555, 0.779, 0.881, 0.930, 0.954, 0.968, 0.977, 0.982, 0.986, 
   0.988, 0.991, 0.992, 0.993, 0.994, 0.995, 0.996, 0.996, 0.996, 0.997, 
   0.997, 0.997, 0.997, 0.998, 0.998, 0.998, 0.998, 0.998, 0.999, 0.998, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 
   0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 0.999, 1.000, 
   1.000, 0.999, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 
   1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000, 1.000},

};
#endif //#define HbtFitter_h
